; =======================================================================
FUNCTION CCF_Validate_Header, $
   header      ; CCF Header

; 06/05/03 I.Yamamura STATUS should be only either 'TEST' or 'SAFE'

   h = header
   if (SIZE(h, /TNAME) NE 'STRUCT') then begin
      lmessage, 1, 'Header is not a STRUCT.'
      error = !FAIL
      return, error
   endif

   if (h.NAME EQ '') then begin
      lmessage, 1, 'NAME undefined.'
      return, !FAIL
   endif

   l = STRLEN(h.NAME)
   if (l LT 4) then begin
      lmessage, 1, 'NAME too short ('+h.NAME+').'
      return, !FAIL
   endif else if (l GT 5) then begin
      lmessage, 1, 'NAME too long ('+h.NAME+').'
      return, !FAIL
   endif else if (l EQ 5) then begin
      ss = STRMID(h.name, 4, 1)
      if ((ss NE 'S') AND (ss NE 'L') AND (ss NE '_')) then begin
         lmessage, 1, '5th char of NAME have to be either S or L or _ ('+h.NAME+').'
         return, !FAIL
      endif
   endif

   if (h.SET_NO LE 0 OR h.REV_NO LE 0) then begin
      lmessage, 1, STRING('SET_NO / REV_NO invalid (', h.SET_NO, ', ', h.REV_NO, ').', /PRINT)
      return, !FAIL
   endif
      
   if ((h.STATUS NE 'TEST') AND (h.STATUS NE 'SAFE')) then begin
      lmessage, 1, 'STATUS invalid ('+h.STATUS+').'
      return, !FAIL
   endif

   if (n_elements(h.RANGE) NE 2) then begin
      lmessage, 1, 'RANGE must have 2 elements.'
      return, !FAIL
   endif

   if (h.RANGE[0] LT 0 OR h.range[1] LT 0) then begin
      lmessage, 1, STRING('RANGE must be 0 or larger (', h.RANGE[0], ',', h.RANGE[1], ').', /PRINT)
      return, !FAIL
   endif

   if ((h.RANGE[1] GT 0) AND (h.RANGE[1] LE h.range[0])) then begin
      lmessage, 1, STRING('RANGE end time must be 0 or larger than start time (', h.RANGE, ').', /PRINT)
      return, !FAIL
   endif

   if (h.CREATOR EQ '') then begin
      lmessage, 1, 'CREATOR is undefined.'
      return, !FAIL
   endif

   if (h.COMMENT EQ '') then begin
      lmessage, 1, 'COMMENT is empty...'
   endif

   return, !SUCCEED
END ; CCF_Validate_Header

; =======================================================================
FUNCTION CCF_Write_Header, $
   lun, $   ; LUN of the CCF file
   header   ; CCF Header structure (assumed to be validated)

   PRINTF, lun, '#HEADER'
   PRINTF, lun, FORMAT='(A-10, A)',      'NAME',    header.NAME
   PRINTF, lun, FORMAT='(A-10, I3.3)',   'SET_NO',  header.SET_NO
   PRINTF, lun, FORMAT='(A-10, I3.3)',   'REV_NO',  header.REV_NO
   PRINTF, lun, FORMAT='(A-10, A)',      'STATUS',  header.STATUS
   PRINTF, lun, FORMAT='(A-10, 2F15.5)', 'RANGE',   header.RANGE
   PRINTF, lun, FORMAT='(A-10, A)',      'CREATOR', header.CREATOR
   PRINTF, lun, FORMAT='(A-10, A)',      'CDATE',   header.CDATE
   PRINTF, lun, FORMAT='(A-10, I3.3)',   'FMTVER',  header.FMTVER

   s = header.COMMENT
   WHILE (s NE '') do begin
      pos = STRPOS(s, STRING(10B))
      if (pos LT 0) then begin
         ss = s
         s = ''
      endif else begin
         ss = STRMID(s, 0, pos)
         l = STRLEN(s)
         if (l GT pos) then s = STRMID(s, pos+1, l-pos-1)
      endelse

      PRINTF, lun, FORMAT='(A-10, A)', 'COMMENT', ss
   ENDWHILE

   PRINTF, lun, '#ENDHEADER'

   return, !SUCCEED
END ; CCF_Write_Header

; =======================================================================
FUNCTION CCF_Data_Definition_String, $
   ccf, $            ; Constant or array data
   FMT=fmt, $        ; Format for data body
   DIM=dim, $        ; Data dimension
   ERROR=error       ; Scolor or array data
                     ; (returned) Data format string

; reset error
   error = !SUCCEED

; Check data dimension
   ndim = SIZE(ccf, /N_DIMENSIONS)
   dim = SIZE(ccf, /DIMENSION)

; Check data type and set data format element
   type = SIZE(ccf, /TNAME)

   CASE type OF
      'BYTE': fmt = 'I'
       'INT': fmt = 'I'
      'UINT': fmt = 'I'
      'LONG': fmt = 'I'
     'ULONG': fmt = 'I'
     'FLOAT': fmt = 'E14.5'
    'DOUBLE': fmt = 'E20.10'
    'STRING': fmt = 'A'
       else : begin
         lmessage, 1, 'Invalid data type ('+type+').'
         error = !FAIL
         return, error 
       end
   ENDCASE

   if (ndim EQ 0) then begin
      defs = STRING('C ', type, /PRINT)
   endif else begin
      defs = STRING('A ', type, dim, /PRINT)
   endelse

   return, defs
END ; CCF_Data_Definition_String

; =======================================================================
FUNCTION CCF_Write_One_Data_Block, $
   lun, $   ; LUN of the CCF file
   ccf      ; CCF data to write

; reset error code
   error = !SUCCEED

; Set data definition string
   defs = CCF_Data_Definition_String(ccf, FMT=fmt_e, DIM=dim, ERROR=error)
   if (error EQ !FAIL) then begin
      lmessage, 1, 'Unable to set CCF data definition.'
      return, error
   endif

; Write definition
   PRINTF, lun, STRING(10B)
   PRINTF, lun, '#DEFINE'
   PRINTF, lun, defs
   PRINTF, lun, '#ENDDEFINE'

; Writing data body 
   ndim = N_ELEMENTS(dim)
   ndat = N_ELEMENTS(ccf)

   PRINTF, lun, STRING(10B)
   PRINTF, lun, '#DATA'

   if (ndat EQ 1) then begin
; Constant

      PRINTF, lun, FORMAT='(' + fmt_e + ')', ccf

   endif else begin
; Array

;;      indices = ARRAY_INDICES(ccf, LINDGEN(ndat))
;;      if (SIZE(indices, /N_DIMENSIONS) EQ 1) then $
;;         indices = REFORM(indices, 1, N_ELEMENTS(indices))
;;      fmt = STRING('(',fmt_e,', "   ; ",',ndim,'I)', /PRINT)
;;
;;      for i = 0L, ndat-1 do begin
;;         PRINTF, lun, FORMAT=fmt, ccf[i], indices[*, i]
;;      endfor

      fmt = STRING('(', ndat, fmt_e, ')')

      PRINTF, lun, FORMAT=fmt, ccf

   endelse

   PRINTF, lun, '#ENDDATA'

   return, !SUCCEED
END ; CCF_Write_One_Data_Block

; =======================================================================
FUNCTION CCF_Write_Structure_Data, $
   lun, $   ; LUN of the CCF file
   ccf      ; CCF data to write

;; 06/05/03 bug fix treating multiple dimension arrays in the structure
;; 06/05/04 further bug fix for structure treatment
;; 06/05/08 further bug fix for structure treatment (single array case)
;;          clean up the code

; Check array size
   if (SIZE(ccf, /N_DIMENSIONS) GT 1) then begin
      lmessage, 1, 'Structure Array have to be 1 dimensional.'
      return, !FAIL
   endif

; Extract TAG names
   tagname_list = TAG_NAMES(ccf)
   ntag = n_elements(tagname_list)

   defs  = STRARR(ntag)
   fmt_e = STRARR(ntag)

; Write definition
   nstruct = N_ELEMENTS(ccf)
   if (nstruct LT 1) then begin
      lmessage, 1, 'No CCF given.'
   endif

   PRINTF, lun, STRING(10B)
   PRINTF, lun, '#DEFINE'
   PRINTF, lun, 'S ', nstruct
   for i = 0L, ntag - 1 do begin
      if (nstruct GT 1) then tccf = ccf[0].(i) else tccf = ccf.(i)
      defs[i] = CCF_Data_Definition_String(tccf, FMT=fmt, DIM=dim, ERROR=error)
      fmt_e[i] = fmt
      if (error EQ !FAIL) then begin
         lmessage, 1, 'Unable to set CCF data definition.'
         return, error
      endif

      PRINTF, lun, '   T ', tagname_list[i], ' ', defs[i]
   endfor
   PRINTF, lun, '#ENDDEFINE'

; Write data body
   for i = 0L, ntag - 1 do begin
      tccf = ccf.(i)

      ndat = N_ELEMENTS(tccf)
      PRINTF, lun, STRING(10B)
      PRINTF, lun, '#DATA'
      PRINTF, lun, '   T ', tagname_list[i]

      if (ndat EQ 1) then begin
; Constant
         PRINTF, lun, FORMAT='('+fmt_e[i]+')', tccf

      endif else begin

         if (nstruct EQ 1) then begin

            fmt = STRING('(', ndat, fmt_e[i], ')')
            PRINTF, lun, FORMAT=fmt, tccf

         endif else begin

            nel = ndat / nstruct
            fmt = STRING('(', nel, fmt_e[i], ')')
            tccf = reform(tccf, nel, nstruct, /OVERWRITE)

            for j = 0L, nstruct-1 do begin
               PRINTF, lun, FORMAT='("; ccf[",I,"]")', j
               PRINTF, lun, FORMAT=fmt, tccf[*, j]
            endfor

         endelse

      endelse

      PRINTF, lun, '#ENDDATA'

   endfor


   return, !SUCCEED
END



; =======================================================================
FUNCTION CCF_Write_CCF_File, $
   ccf_file, $             ; CCF File Name
   ccf, $                  ; CCF Data
   header, $               ; CCF Header 
   OVERWRITE=overwrite     ; Allow to overwrite existing file.
                           ; (returned) Error status

; reset error codes.
   error = !SUCCEED

; Check Header
   error = CCF_Validate_Header(header)
   if (error EQ !FAIL) then begin
      lmessage, 1, 'Invalid Header.'
      error = !FAIL
      return, error
   endif

; Check if CCF data really exist
   if (SIZE(ccf, /TNAME) EQ 'UNDEFINED') then begin
      lmessage, 1, 'CCF is undefined.'
      error = !FAIL
      return, error
   endif

; Start Writing to file
; Check if there is a file with the same name
   if (NOT Keyword_Set(OVERWRITE) AND (FILE_TEST(ccf_file) EQ 1)) then begin
      lmessage, 1, 'File "'+ccf_file+'" exists.'
      lmessage, 1, 'Use /OVERWRITE option if you really want to overwrite.'
      error = !FAIL
      return, error
   endif

; open the CCF file in write mode
   OPENW, lun, ccf_file, /Get_LUN, ERROR=error
   if (error NE 0) then begin
      msg = STRING(FORMAT='(A, A, A, I, A, A)', $
         'Error to open CCF file (', ccf_file, '), error = ', error, !ERROR_STATE.msg,'.')
      lmessage, 1, !ERROR_STATE.msg
      error = !FAIL
      return, error
   endif

; Set Time Stamp
   CALDAT, SYSTIME(/JULIAN), mm, dd, yy, hh, ms, ss
   header.CDATE = STRING(FORMAT='(I4.4,"-",I2.2,"-",I2.2,"T",I2.2,":",I2.2,":",I2.2)', $
      yy, mm, dd, hh, ms, ss, /PRINT)

; Set FMTVER
   header.FMTVER = CCF_Current_FMTVER()

; Write Header
   error = CCF_Write_Header(lun, header)

; Write data definition and data body

   if (SIZE(ccf, /TNAME) EQ 'STRUCT') then begin
      error = CCF_Write_Structure_Data(lun, ccf)
   endif else begin
      error = CCF_Write_One_Data_Block(lun, ccf)
   endelse

   FREE_LUN, lun
   
   RETURN, error

END ; CCF_Write_CCF_File
