; $Id: gb_defringe.pro,v 1.1 2006/05/12 00:51:38 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	GB_defringe.pro
; PURPOSE:
;	detect and correct for periodic noise
; CATEGORY:
;	Function. Green Box procedure in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = GB_defringe( tsd, smooth=smooth,
;	[, CAL_CONST_FILE = cal_const_file,
;	    fringelist=fringelist, pixel=pixel, plot=plot, $
;	    _extra=_extra] )
; INPUTS:
;	tsd		TSD object reference.
;	smooth		'noise-free' representation of the flux.
;			from gb_set_glitch_status_gp
;	cal_const_file	Calibration constants file;
;			format, usage are TBD (optional.)
;	pixel		operate on these pixels only
;			default: all
;	plot		make plots
;	_extra		for plotting.
; OUTPUTS:
;	tsd		TSD object reference. Deglitched TSD.
;	fringelist	list of detected fringes.
;			How to manage the summaries is under considerations.
;RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;	See README
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Oct. 2002		Written by A. Kawamura.
;	Dec. 2002		Header is added.
;	2003-2005   DK		developments
;	June 2005   DK		make default method = 2
;-

FUNCTION GB_defringe, tsd, smooth=smoflux, Calib_Const_File = ccf, $
	    fringelist=fringelist, pixel=pixel, plot=plot, $
	    _extra=_extra

;   ON_ERROR 2

    syst = systime(1)

;;; Starting GB_Defringe ;;;
    lmessage, 7,'Executing GB_Defringe...'

    if ( n_elements( fringelist ) GT 0 ) then begin
	if ( size( fringelist, /type ) EQ 7 ) then listfile = fringelist $
	else begin
	    infile = tsd->get_filename()
	    str_len = strpos( infile, ".fits" )
	    listfile = strmid( infile, 0, str_len ) + '_fringe_list.dat'
	endelse
	openw, glunit, listfile, /get_lun
    endif else if ( keyword_set( silent ) ) then $
	openw, glunit, '/dev/null', /get_lun

    pltype = size( plot, /type )
    if ( pltype GT 0 ) then begin
	do_plot = 1
	if ( pltype EQ 7 ) then start_hc, plot $
	else do_plot = plot
    endif

;    if ( n_elements( ccf ) EQ 0 ) then ccf = 'DEFR_'
;    frincal = ccf_read_ccf( ccf, tsd, error=err )
;    if ( err EQ !FAIL ) then $
;	message, "No CCF file found for fringe removal", /continue

    type = tsd->get_datatype()
    if ( type EQ 'FIS_SW' ) then begin
	detstep = 20
	if ( n_elements( pixel ) EQ 0 ) then pixel = indgen( 100 ) + 1 $
	else detstep = 1
    endif
    if ( type EQ 'FIS_LW' ) then begin
	detstep = 15				      ;; FIXME FIXME
	if ( n_elements( pixel ) EQ 0 ) then begin
	    pixel = indgen( 75 ) + 1
	    pixel = where( pixel NE  3 OR pixel NE 18 OR $	; really dead
			   pixel NE 42 OR pixel NE 63 ) + 1
	endif else detstep = 1
    endif
    nfringes = 1
    nresetstep = 1
    npixel = n_elements( pixel )

    flux = fringes( tsd, pixel, nfringes=nfringes, smooth=smoflux, $
	    resetstep=resetstep, detstep=detstep, unit=glunit, $
	    plot=do_plot, status=status, _extra=_extra )

    if ( status EQ 0 ) then $
	status = tsd->set_value_tsd( 'fis_obs', 'flux', flux )

    if ( n_elements( fringelist ) GT 0 OR keyword_set( silent ) ) then $
	free_lun, glunit
    if ( pltype EQ 7 ) then stop_hc

;;; Write HISTORY in header ;;;

    lmessage, 7, 'Writing HISTORY',/CONTINUE
    hist_message = ' gb_defringe.pro,v 0.0 done. CCF= '

    IF KEYWORD_SET(ccf) NE 1 THEN BEGIN
	hist_message = hist_message + 'NONE'
    ENDIF ELSE BEGIN
	hist_message = hist_message + ccf
    ENDELSE

    dummy = tsd_fxaddhist(tsd, hist_message)
    IF dummy NE 0 THEN $
	MESSAGE,'Unable to write HISTORY',/CONTINUE

;    print, systime(1) - syst

    return, status

END


