; $Id: gb_get_fbad_arr.pro,v 1.4 2006/01/18 06:57:00 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;       gb_get_fbad_arr.pro
; PURPOSE:
;       The flags of the TSD are read and an array which states 
;       one/zero if any of the flags are/are not set for at
;       the corresponding index of an anonymous chunnel.
; CATEGORY:
;	Function. Green Box library for ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = gb_get_fbad_arr(tsd[, /no_flag_rst])
; INPUTS:
;	tsd		TSD object reference.
;	(no_flag_rst	Should be stated if the flags for reset 
;                       are to be ignored. 
; OUTPUTS:
;	f_val           Byte array which states one/zero 
;                       if any of the flags are/are not set for at
;                       the corresponding index of an anonymous channel. 
; RETURN:
;	Byte Array of bad flag information. 1 for with flag, 0 for no flag
; RESTRICTIONS:
;	NONE	
; PROCEDURE:
;	1. Read TSD; detector data for array size, 
;          as well as flag and status
;	2. Read flags for "detector condition,";
;		bad record for N- and W-band(badrcdn, badrcdw),
;		blank record(blank), in SAA(in_saa), and 
;		Moon(near_moon).
;	3. Read flags for "detector pixel condition|,";
;		blank, dead pixel(dead), saturation(saturate),
;		and after reset(afterrst).
;	4. Check if the reset flags to be included or not.
;	5. Create and return byte array
;
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2003 Dec. 1, "ASTRO-F/FIS Survey Data Structure"
;       Kawamura et al. 2003 Jan. "ASTRO-F data reduction pipeline -green box-"
;
; MODIFICATION HISTORY:
; 	Oct. 2003      Written by A. Kawamura.
;	Dec. 2003      TSD ver2 implemented
;       Nov. 2004      TSD3
;-

FUNCTION gb_get_fbad_arr, tsd, no_flag_rst=no_flag_rst
;ON_ERROR 2
IF N_PARAMS(0) LT 1 OR N_PARAMS(0) GT 3 THEN BEGIN
MESSAGE, '  Calling Sequence: gb_get_fbad_arr(tsd[,/no_flag_rst])',/CONTINUE
MESSAGE, '       tsd = TSD object reference',/CONTINUE
MESSAGE, '       when /no_flag_rst is set, reset flag is ignored',/CONTINUE
    return, !FAIL;
ENDIF

MESSAGE,'Getting array for flags...',/CONTINUE
;;; Extract values from TSD object reference ;;;
	vdet=tsd->get_value_tsd('fis_obs','flux')
  ;;; number of channels ;;;
	si = SIZE(vdet)
	num_ch = si[1]
	num_data = si[2]
  ;;; create fbad_arr
        fbad_arr=bytarr(num_ch, num_data)

;;; Check flags

    f_blank=tsd->get_flag_tsd('fis_obs','blank')        ;; 0 for usable data
    f_insaa=tsd->get_flag_tsd('fis_obs','in_saa')
    f_nmoon=tsd->get_flag_tsd('fis_obs','near_moon')
;    f_badrcdn=tsd->get_flag_tsd('fis_obs','badrcdn')
;    f_badrcdw=tsd->get_flag_tsd('fis_obs','badrcdw')

    ok_arr=f_blank+f_insaa+f_nmoon ;; 0 for usable data

       dummy = findgen(num_ch, num_data)
    FOR n = 0, num_ch-1 DO BEGIN   
       dummy[n,*]=ok_arr[*]
    ENDFOR
       ok_arr=dummy

;;; Check pix_flags

    f_dead=tsd->get_pix_flag_tsd('fis_obs','dead')
    f_bad=tsd->get_pix_flag_tsd('fis_obs','bad')
    f_saturate=tsd->get_pix_flag_tsd('fis_obs','saturate')
;    f_spike=tsd->get_pix_flag_tsd('fis_obs','spike')
    f_aft_reset=tsd->get_pix_flag_tsd('fis_obs','reset')
    f_no_dif=tsd->get_pix_flag_tsd('fis_obs','no_diff')

IF KEYWORD_SET(no_flag_rst) EQ 1 THEN BEGIN
;    ;;; Check if BAD is from reset or from elsewhere 


;    ind_bad=WHERE(f_bad NE 0, cnt_bad)
;    IF cnt_bad NE 0 THEN BEGIN
;      ind_rst=WHERE(f_aft_reset NE 0, cnt_rst)
;      IF cnt_rst NE 0 THEN BEGIN
;        dummy1=f_dead+f_saturate+f_spike+ok_arr
;        ind_dummy1=WHERE(dummy1 NE 0, cnt_dummy1)   ;; flag other thab BAD/AFTERRST
;        IF cnt_dummy1 NE 0 THEN dummy1[ind_dummy1]=1
;        dummy2=fltarr(num_ch,num_data)
;        dummy2[ind_rst]=-1
;        ind_only_rst=WHERE(dummy1+dummy2 EQ -1, cnt_only_reset)
;        msg=STRCOMPRESS('Flags, BAD, only due to reset are '+STRING(cnt_only_reset)+'pts.')
;        MESSAGE,msg,/CONTINUE        
;        IF cnt_only_reset NE 0 THEN f_bad[ind_only_rst]=0
;      ENDIF
;    ENDIF

;    ok_pix_flag=f_dead+f_saturate+f_bad+ok_arr
ENDIF ELSE BEGIN
    ok_pix_flag=f_dead+f_bad+f_saturate+f_aft_reset+f_no_dif+ok_arr
ENDELSE

   ind_nook=WHERE(ok_pix_flag NE 0, cnt_nook)
   IF cnt_nook NE 0 THEN ok_pix_flag[ind_nook]=1
   fbad_arr=ok_pix_flag

    return, fbad_arr
END
