; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   LMessage.pro
;
; PURPOSE:
;   LMessage is a wrapping procedure to IDL's MODULE to
;   control verboseness of message output. It is used
;   with a (local) system variable !MSG_LEVEL.
;
; CATEGORY:
;   Library
;
; CALLING SEQUENCE:
;   LMESSAGE, level, text, [/PRINT] [, options for MESSAGE]
;
; INPUTS:
;   level   Message level
;   text    Message body
;   See help of MESSAGE procedure for options.
;
; OUTPUTS:
;   Message text on screen
;
;   /PRINT: Use PRINT to output message instead of MESSAGE.
;
; RETURN:
;   none
;
; RESTRICTIONS:
;   LMessage assumes presence of !MSG_LEVEL.
;   Without this variable an unexpected result may take place.
;   (In the FIS data reduction system it is set in fisdr.pro)
;
;   LMessage uses an option of MESSAGE, LEVEL=level internally
;   in order to print procedure name which calls LMessage.
;   Duplicated set of this option at the call cause an  unexpected result.
;
; PROCEDURE:
;   When LMessage is called, it compares level and current value
;   of !MSG_LEVEL. It prints message text when level is equal or
;   smaller than !MSG_LEVEL. If !MSG_LEVEL is has not been defined
;   the message is always printed.
;
;  The default !MSG_LEVEL is 5.
;
;   LMessage does not check 'validness' of level parameter.
;   The following is the guideline.
;
; level  Purpose  Purpose
;   0    Critical Critical error that stops the process immediately.
;   1    Error    Serious error. Process can continue but the results
;                 are likely wrong.
;   3    Warning  Process can continue but the results need to be checked.
;   5    Notice   Important notification. e.g., start /end of procedure,
;                 CCF file used, essential decision in the program, etc.
;
;   7    Info     Verbose Informational messages.
;   9    Debug    Messages for debugging purpose
;
;   LMessage terminates process (only) when level 0 message appears. 
;   Otherwise, it continues processing.
;
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;   2005-06-14 Ver. 1.0    IY    First version.
;   2005-06-28 Ver. 1.1    IY    Check if system variable !MSG_LEVEL exists.
;                                Add /PRINT option
;   2005-11-30 Ver. 1.2    IY    Process terminated only at level 0 message
;-
; ==============================================================
PRO LMessage, $
   level, $             ; Error level of this message
   text, $              ; Message text
   PRINT=print, $       ; Use PRINT to output message
   _EXTRA=extra         ; anything else passed to MESSAGE

; Message wrapped by level control
; 05/06/14 I.Yamamura
; 05/06/27 I.Yamamura check presence of !MSG_LEVEL

   DEFSYSV, '!MSG_LEVEL', EXISTS = e
   if (e) then if (level GT !MSG_LEVEL) then RETURN

   if (Keyword_set(PRINT)) then PRINT, text, _EXTRA=extra $
   else                         MESSAGE, text, LEVEL=-1, /CONTINUE, _EXTRA=extra
   if (level LE 0) then MESSAGE, LEVEL=-1, "Process terminated."

   RETURN

END
