; $Id: gb_conv_to_volt.pro,v 1.8 2006/07/15 10:52:13 makiuti Exp $
;
; Copyright (c) 2003-2006, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   GB_Conv_to_Volt.pro
;
; PURPOSE:
;   The input data is converted to Voltage unit.
;
; CATEGORY:
;   Function. Green Box procedure in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = GB_Conv_to_Volt(tsd
;            [, ccf_thresh = ccf_thresh, /FERR])
;
;    ;; Default values of ccf_thresh is 1
;    ;; Default values of FERR is 1 (Calculate rms of Flux, and set FErr)
;
; INPUTS:
;   tsd     TSD object reference.
;   cal_const_file    Calibration constants file;
;
; OUTPUTS:
;   tsd     TSD object reference with 'Flux' in Volt.
;
; RETURN:
;   !SUCCEED if function ended successfully.
;   !FAIL in failure.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   1. Detector data is read.
;   2. Calibration constants file is read.
;   3. Devided by 6 for co-added mode
;   4. Conversion factor is determined; the conversion factor
;      should depend on the detector output.
;   5. Conversion is carried out.
;   6. FErr (rms) is calculated and set.
;   7. Converted TSD with history in its header is written.
;
; EXAMPLE:
;
; REFERENCE:
;
;
; MODIFICATION HISTORY:
;   Nov. 2003     Written by A. Kawamura.
;   Dec. 18, 2003 IY  Unit conversion (cal_vdet) is done as double.
;   Aug. 2004     Updated for TSD3
;   Oct. 2004     IT#2
;   Jul. 2006     Last modified by Makiuti
;-

FUNCTION gb_conv_to_volt, tsd, ccf_thresh=ccf_thresh, FERR=ferr

;ON_ERROR 2

;;; Starting gb_conv_to_volt ;;;
    LMESSAGE, 5, 'Executing GB_Conv_to_Volt...', /CONTINUE

;;; Extract values from TSD object ;;;
    det = tsd->Get_value_tsd('FIS_OBS','Det')
    vdet = DOUBLE(det)

  ;;; number of channels ;;;
    si = SIZE(vdet)
    num_ch = si[1]
    num_data = si[2]

    ;;-----------------------
    ;; Check packetid and divide data for co-added mode.
    ;;-----------------------
    packetid = tsd->Get_value_tsd('FIS_OBS','PacketId')
    aftime = tsd->Get_value_tsd('FIS_OBS','AFTIME')
    inst = tsd->Get_header_keyword_tsd('PRIMARY','INSTRUME')

    ; Get SEQ (sampling) pattern indicator array
    seq_mode = smpl_mode(packetid, AFTIME=aftime[0], TYPE=inst)

    ; For 6-times co-add mode
    ind = WHERE(seq_mode EQ 2, cnt) ; Co-add mode

    IF (cnt NE 0) THEN BEGIN
      LMESSAGE, 7, "Dividing by 6 for co-added mode", /CONTINUE
      vdet[*,ind] = vdet[*,ind]/6.D
    ENDIF

    ; For high speed sample (quadruple co-add) mode for maneuver operation
    ind = WHERE(seq_mode EQ 5, cnt) ; Co-add mode

    IF (cnt NE 0) THEN BEGIN
      LMESSAGE, 7, "Dividing by 4 for maneuver mode", /CONTINUE
      vdet[*,ind] = vdet[*,ind]/4.D
    ENDIF

    ;;-----------------------
    ;; Conversion ADU to Volt
    ;;-----------------------
    ind1 = WHERE(vdet GE -35157., cnt1)
    IF (cnt1 GT 0) THEN $
      vdet[ind1] = -9.34D-5 + 1.416D-6*vdet[ind1]

    ind2 = WHERE((vdet GE -64500.D) AND (vdet LT -35157.D), cnt2)
    IF (cnt2 GT 0) THEN $
      vdet[ind2] = 9.903D-3 + 1.704D-6*vdet[ind2]

    ind3 = WHERE((vdet GE -72008.D) AND (vdet LT -64500.D), cnt3)
    IF (cnt3 GT 0) THEN $
      vdet[ind3] = 0.759D + 1.332D-5*vdet[ind3]

    ind4 = WHERE((vdet GE -72976.D) AND (vdet LT -72008.D), cnt4)
    IF (cnt4 GT 0) THEN $
      vdet[ind4] = 7.239D + 1.033D-4*vdet[ind4]

    ind5 = WHERE(vdet LT -72976.D, cnt5)
    IF (cnt5 GT 0) THEN $
      vdet[ind5] = -0.3D


;;----------------------------------------------------------
;; Write converted data and set flags and qualities
;;----------------------------------------------------------

  ret = tsd->Set_value_tsd('FIS_OBS','Flux', vdet)

    ;;-----------------------
    ;; Check the range of conversion using CCF
    ;;-----------------------
    ;;; Check the range of voltage.
    ;  Thresholds are independent of channel and detector(SW or LW).

    msg = "Setting flag and quality for data with out of range."
    LMESSAGE, 7, msg, /CONTINUE

    IF (N_elements(ccf_thresh) EQ 0) THEN ccf_thresh = 1 ;#default value

    ;; Get thresholds
    IF (ccf_thresh NE 0) THEN BEGIN ;; Read CCF and set flags

      ;;; read CCF ;;;
      LMESSAGE, 7, "Read CCF for ADU to Volt Conversion range thresholds.", /CONTINUE
      gb_c2v_thsh_ccf = CCF_Read_CCF('C2VT_', tsd, HEADER=hdr, ERROR=err)

      ;;; Set thresholds of high and low
      thresh0 = gb_c2v_thsh_ccf[0] ; Lowest threshold of conversion range in Volt
      thresh1 = gb_c2v_thsh_ccf[1] ; Threshold of unreliable range in Volt
      thresh2 = gb_c2v_thsh_ccf[2] ; Threshold of reliable (linear) range in Volt
      thresh3 = gb_c2v_thsh_ccf[3] ; Highest threshold of conversion range in Volt

    ENDIF ELSE BEGIN ;; Dose not read CCF, use dummy thresholds

      thresh0 = -1000.
      thresh1 = -1000.
      thresh2 = -1000.
      thresh3 = 1000.

    ENDELSE

    ;; Display just for information
    msg = ["ADU to Volt Conversion range thresholds : ", string([thresh0, thresh1, thresh2, thresh3]), "Volt"]
    LMESSAGE, 7, msg, /PRINT

    ;;-----------------------
    ;; Set "bad" flag and quality indicators
    ;;-----------------------
    fbad = tsd->Get_pix_flag_tsd('FIS_OBS','Bad')
    qual = tsd->Get_quality_tsd('FIS_OBS','Qual_cv_param')

    ind = WHERE(vdet GT thresh3, cnt)
    IF (cnt NE 0) THEN BEGIN
      fbad[ind] = 1
      qual[ind] = 3
      msg = string(cnt)+" points are above upper threshold value."
      LMESSAGE, 3, msg, /CONTINUE
    ENDIF

    ind = WHERE(vdet LT thresh0, cnt)
    IF (cnt NE 0) THEN BEGIN
      fbad[ind] = 1
      qual[ind] = 3
      msg = string(cnt)+" points are below lower threshold value."
      LMESSAGE, 3, msg, /CONTINUE
    ENDIF

    ind = WHERE((vdet GT thresh0) AND (vdet LE thresh1), cnt)
    IF (cnt NE 0) THEN BEGIN
      qual[ind] = 2
      msg = string(cnt)+" points are out of reliable range."
      LMESSAGE, 3, msg, /CONTINUE
    ENDIF

    ind = WHERE((vdet GT thresh1) AND (vdet LE thresh2), cnt)
    IF (cnt NE 0) THEN BEGIN
      qual[ind] = 1
      msg = string(cnt)+" points are out of linear range."
      LMESSAGE, 3, msg, /CONTINUE
    ENDIF

    ret = tsd->Set_pix_flag_tsd('FIS_OBS','Bad', fbad)
    ret = tsd->Set_quality_tsd('FIS_OBS','Qual_cv_param', qual)

    ;;-----------------------
    ;; Set flag of "arith_err"
    ;;-----------------------
    msg="Setting flag, arith_err"
    LMESSAGE, 7, msg, /CONTINUE

    ind_nan = WHERE(FINITE(vdet, /NAN), count_nan)
    ind_inf = WHERE(FINITE(vdet, /INFINITY), count_inf)

    msg = STRCOMPRESS('Number of NaN ... '+STRING(count_nan))
    LMESSAGE, 7, msg, /CONTINUE
    msg = STRCOMPRESS('Number of INFINITE ... '+STRING(count_inf))
    LMESSAGE, 7, msg, /CONTINUE

    arith_err = bytarr(num_ch,num_data)
    IF (count_nan NE 0) THEN arith_err[ind_nan] = 1
    IF (count_inf NE 0) THEN arith_err[ind_inf] = 1
    ind = WHERE(arith_err NE 0, cnt)
    IF (cnt NE 0) THEN arith_err[ind] = 1
    ret = tsd->Set_pix_flag_tsd('FIS_OBS','Arith_err', arith_err)

    ;;-----------------------
    ;; Set flag of "bad" for arith_err + previous bad
    ;;-----------------------
    msg = "Setting flag, bad..."
    LMESSAGE, 7, msg, /CONTINUE

    fbad = tsd->Get_pix_flag_tsd('FIS_OBS','Bad')

    dmy_bad = arith_err + fbad
      ind = WHERE(dmy_bad NE 0, cnt)
      IF (cnt NE 0) THEN dmy_bad[ind]=1
    ret = tsd->Set_pix_flag_tsd('FIS_OBS','Bad', dmy_bad)

;;----------------------------------------------------------
;; Set 'FERR'
;;----------------------------------------------------------

  IF (n_elements(ferr) EQ 0) THEN ferr = 1 ;#default value

  IF (ferr EQ 1) THEN BEGIN
    LMESSAGE, 7, 'Deriving errors in flux values...', /CONTINUE

    flux = tsd->Get_value_tsd('FIS_OBS','Flux')
    ferr = tsd->Get_value_tsd('FIS_OBS','Ferr')

    badf = tsd->Get_pix_flag_tsd('FIS_OBS','Bad')
    rstf = tsd->Get_pix_flag_tsd('FIS_OBS','Reset')


    FOR ch = 0, num_ch-1 DO BEGIN ; Start of a channel roop

      ;--------------------
      ; Get an index of reset frame
      rst_ind = where(rstf[ch, *] EQ 1, cnt)

      end_frame_num = n_elements(badf[0,*])-1

      ; In the case of CDS
      IF (cnt EQ 0) THEN BEGIN
        rst_ind = [0, end_frame_num]
        cnt = 2
      ENDIF

      ;; Regard the first and end frames as reset
      IF (rst_ind[0] NE 0) THEN BEGIN
        ; Add the first frame number
        rst_ind = [0, rst_ind]
        cnt = cnt+1
      ENDIF

      IF (rst_ind[cnt-1] NE end_frame_num) THEN BEGIN
        ; Add a frame number of end of the file
        rst_ind = [rst_ind, end_frame_num]
        cnt = cnt+1
      ENDIF
      ;--------------------

      ; Set a number of frames which are extracted to calculate the noise level.
      calc_num = 6 ; #This value is to be revised if needed,

      ;;
      FOR i = 0, cnt-2 DO BEGIN ; Start of a reset roop

        FOR j = rst_ind[i], rst_ind[i+1] DO BEGIN ; Start of a roop in a block of reset

          s_frm = j - Round(calc_num/2) ; Start frame number to estimate an error for 'j'th frame
          e_frm = j + Round(calc_num/2) ; End frame number

          ; For frames of just before and after reset
          IF (s_frm LT rst_ind[i])   THEN s_frm = rst_ind[i]     & e_frm = s_frm + calc_num
          IF (e_frm GE rst_ind[i+1]) THEN e_frm = rst_ind[i+1]-1 & s_frm = e_frm - calc_num

          IF (s_frm LT 0) THEN s_frm = 0
          IF (e_frm GT end_frame_num) THEN e_frm = end_frame_num

          ; Extract data for 'j'th frame
          y    = flux[ch, s_frm:e_frm]
          bady = badf[ch, s_frm:e_frm]

          oky_ind = where(bady EQ 0, cnt_ok)
          IF (cnt_ok GT 0) THEN BEGIN
            y = y[oky_ind]
          ENDIF ELSE BEGIN
            continue
          ENDELSE

          dt_num = n_elements(y)

          IF (dt_num GE 4) THEN BEGIN ; Fiting by quadratic function
            x = indgen(dt_num)

            par = poly_fit(x,y,2)
            f = par[0]+par[1]*x+par[2]*x^2 ; fitted function

            ; Calculate a deviation around the fitted curve
            ferr[ch, j] = sqrt(total((y-f)^2)/(dt_num-1))
          ENDIF ELSE IF (dt_num GE 2) THEN BEGIN ; Without fitting
            ferr[ch, j] = stddev(y)
          ENDIF ELSE BEGIN ; Not calculate deviation
            ferr[ch, j] = 0
          ENDELSE

        ENDFOR ; End of a roop in a block of reset.

      ENDFOR ; End of a reset roop

    ENDFOR ; End of a channel roop

    ferr = float(ferr)
    ret = tsd->Set_value_tsd('FIS_OBS','Ferr', ferr)

  ENDIF

;;----------------------------------------------------------
;; Write a history in TSD header
;;----------------------------------------------------------
    LMESSAGE, 7, 'Writing HISTORY ', /CONTINUE
    name_ver = gb_get_name_ver(infile='gb_conv_to_volt')
    hist_message = STRCOMPRESS(' '+name_ver+' done. CCF= ')

    dummy = tsd_fxaddhist(tsd, hist_message, /prres)
    IF (dummy NE 0) THEN $
      LMESSAGE, 3, 'Unable to write HISTORY', /CONTINUE

    RETURN, !SUCCEED

END
