; $Id: fit_cubic_splines.pro,v 1.4 2006/05/11 05:39:46 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	fit_cubic_splines.pro
; PURPOSE:
;	fir a smoothing cubic spline through data points
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	yfit = fit_cubic_splines( x, y, knots, weight=weight, $
;			logpr=logpr, robust=robust, knotsep=knotsep )
;
; INPUTS:
;	x	    x data. independent axis
;	y	    y data. dependent axis
;	knots	    number of evenly spaced x-locations of the spline knots
;	weight	I   weights of the datapoints
;		    default : all 1.
;		O   weights as derived from robust fitting
;	prior	    prior for evidence calculation
;		    default : 10000.0
;	robust	    number of iteration to robustify the solution
;		    default 0;
;	knotsep     governs the order of the polynomial to fit when knots=2
;
;
; OUT:
;	logpr	    log of the evidence
; RETURN
;	yfit	    the (smooth) spline fit
;
; RESTRICTIONS:
;	knots has to minimally 2
; PROCEDURE:
;
;
;
;
;
; EXAMPLE:  x = findgen( 20 )
;	    y = exp( -0.3 * x ) + 0.02 * randomn( seed, 20 )
;	    yfit = fit_cubic_splines( x, y, 5 )
;	    fits a 5 knots cubic spline through these points
; REFERENCE:
;
; MODIFICATION HISTORY:
;	Nov. 2005		Written by Do Kester
;-

FUNCTION fit_cubic_splines, time, flux, knots, weight=weight, $
			logpr=logpr, prior=prior, robust=robust, knotsep=knotsep

FORWARD_FUNCTION bayesfit

npt = n_elements( flux )
node = time[0] + (findgen( knots ) * ( time[npt-1] - time[0] )) / ( knots-1 )

wgt = ( n_elements( weight ) NE npt ) ? dblarr( npt ) + 1 : weight
if ( n_elements( robust ) EQ 0 ) then robust = 0
if ( n_elements( prior ) EQ 0 ) then prior = 10000.0
if ( n_elements( knotsep ) EQ 0 ) then knotsep = 40

; set up the design matrix
if ( knots EQ 2 ) then begin
    if ( npt LT knotsep/8 ) then np = 1 $
    else if ( npt LT knotsep/4 ) then np = 2 $
    else if ( npt LT knotsep/2 ) then np = 3 $
    else np = 4
endif else np = knots + 2
desmat = dblarr( np, npt )

; first handle the exception where knots equals 2
tt = 1
if ( np LE 3 ) then for k = 0, np-1 do begin
    desmat[k,*] = tt
    tt *= time
endfor else begin

; now the real stuff: knots > 2
; some quite opaque algebra is behind this

    s0 = dblarr( np )			    ; sums x^0 coeff
    s1 = dblarr( np )			    ; sums x^1 coeff
    s2 = dblarr( np )			    ; sums x^2 coeff
    s0[0] = 1
    s1[1] = 1
    s2[2] = 1

    i = 3
    for k = 1, knots - 1 do begin
	b = where( time GE node[k-1] AND time LE node[k], nb )

	if ( nb EQ 0 ) then continue

	x1 = ( time[b] - node[k-1] )/( node[k] - node[k-1] )  ; unit intervals
	x2 = x1 * x1					      ; 2nd moment
	x3 = x2 * x1					      ; 3rd moment

	desmat[0,b] = s0[0]
	desmat[1,b] = s0[1] + s1[1]*x1
	desmat[2,b] = s0[2] + s1[2]*x1 + s2[2]*x2
	for j = 3, i-1 do begin
	    desmat[j,b] = s0[j] + s1[j]*x1 + s2[j]*x2
	    s0[j] = s0[j] + s1[j] + s2[j]
	    s1[j] = s1[j] + 2 * s2[j]
	endfor
	desmat[i,b] = x3
	s0[i] = 1
	s1[i] = 3
	s2[i] = 3
	i = i + 1

	s0[1] = s0[1] + s1[1]
	s0[2] = s0[2] + s1[2] + s2[2]
	s1[2] = s1[2] + 2
    endfor
endelse

wt = wgt
for k = 0, robust do begin

    dw = desmat * ( wt ## ( dblarr( np ) + 1 ) )	; weighted version
    fw = flux * wt					; weighted flux

    mat = dw # transpose( dw )				; normal matrix
    vec = dw # fw					; vector

    if ( max( abs( mat ) ) EQ 0 ) then begin
	logpr = 0
	return, make_array( size=size( flux ), value = 0 )
    endif

    stdev = dblarr( np + 1 ) + prior		    ; priors
    chisq = total( double( fw^2 ) )		    ; sum over (wgt*flux)^2
    slsigma = total( wt )			    ; all data same weight (=1)
    scale = 0					    ; automatic scaling

    if ( bayesfit( mat, vec, slsigma, npt, chisq=chisq, sol=sol, stdev=stdev, $
		scale=scale, logpr=logp ) NE 0 ) then break

    if ( k EQ 0 ) then logpr = logp	    ; I have to think more about this

    result = sol # desmat

    if ( k GE robust ) then break

; recalculate the weigths based on the misfit (Tukey weighting)
    dif = flux - result
    tukey = 1.0 / ( scale * 6 )
    wt = ( ( abs( dif * tukey ) < 1 ) ^ 2 - 1 ) ^ 2
    wt *= wgt

endfor

; return the weights
weight = wt

RETURN, result
END
