; $Id: im_destripe.pro,v 1.5 2006/05/11 05:39:46 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	im_destripe.pro
; PURPOSE:
;	destripe one detector scan
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	Result = im_destripe( flux, flag, x, y, mapset, mapinfo, $
;	    det, scannr, plot=plot, _extra=_extra )
;
; INPUTS:
;	flag		pertaining flags
;	x, y		projected coordinates of the samples
;	mapinfo 	map information
;	mapset		maps
;	det		detector number of the samples
;	scannr		scan number of the samples
;	plot		produce plots
;	_extra		for plotting
;
; IN/OUT:
;	flux		update of valid flux for this det
; RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;
; PROCEDURE:
;	For each sample find a series of mapinfo.width pixels perpendicular
;	to the scan direction. Determine the median of them.
;	For all samples determine the difference between their fluxes and
;	the median. Fit a smoothing cubic spline to these differences.
;	At each location where a glitch has been detected, try a
;	discontinuity in the spline fit (to fit a possible tail).
;	If the evidence for a tail is larger than without, keep the tail fit.
;
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Oct. 2004		Written by Do Kester
;-

FUNCTION im_destripe, flux, flag, x, y, mapset, mapinfo, $
	    det, scannr, plot=plot, _extra=_extra

COMPILE_OPT IDL2

COMMON plotopts, manual

;   ON_ERROR 2
    b = where( flag NE 1, nb )

    kk = fix( x[b] )
    jj = fix( y[b] )
    bb = where( jj GE 0 AND jj LT mapinfo.pixels[1] AND $
		kk GE 0 AND kk LT mapinfo.pixels[0], nbb )

    if ( nbb EQ 0 ) then return, !SUCCEED

    jb = jj[bb]
    kb = kk[bb]
    indx = b[bb]
    ind0 = indx - indx[0]

    g = where( flag[indx] GE 2, ng )

    fb = flux[indx]

    wid = mapinfo.width;
    rob = mapinfo.robust
    box = mapinfo.box
    ksep = mapinfo.knotsep

    m0 = min( flux[indx], max=m1 )

    if ( keyword_set( plot ) ) THEN BEGIN
	plot, indx, fb, ps=1, position=[0.1,0.3,0.9,0.9], yr=[0<m0,1>m1],$
	  title=string( "Scan ",scannr,", det ", det, format="(a,i3,a,i3)"), $
	  _extra=_extra
	if ( !D.name EQ 'X' ) then begin
	    tv, rebin( mapset.coadd, 120,120 )*100, 0
	    ff = 120.0 / mapinfo.pixels[0]
	    plots, kb*ff, jb*ff+!d.y_size-120, /device
	endif
;	plots, kb[0]*ff, jb[0]*ff+!d.y_size-120, /device
;	plots, kb[nbb-1]*ff, jb[nbb-1]*ff+!d.y_size-120, /device, /continue
	if ( ng GT 0 ) then oplot, indx[g], flux[indx[g]], ps=4, color=4
    endif

;    fc = mapset.coadd[kb,jb]
    dj = ( x[b[nb-1]] - x[b[0]] ) / nb
    dk = ( y[b[nb-1]] - y[b[0]] ) / nb

    foot = dblarr( nbb, wid )
    over = dblarr( nbb, wid )
    kq = kb - dk * wid / 2
    jq = jb + dj * wid / 2

    for i = 0, wid - 1 do begin
	q = where( jq GE 0 AND jq LT mapinfo.pixels[1] AND $
		   kq GE 0 AND kq LT mapinfo.pixels[0], nq )
	if ( nq GT 0 ) then begin
	    foot[q,i] = mapset.coadd[kq[q],jq[q]]
	    qq = where( foot[q,i] NE 0, nqq )
	    if ( nqq GT 0 ) then over[q[qq],i] = 1
	    if ( keyword_set( plot ) ) THEN $
		oplot, indx[q], foot[q,i], ps=3, color=2
	endif
	kq += dk
	jq -= dj
    endfor

    fc = dblarr( nbb )
    for k = 0, nbb-1 do begin
	q = where( over[k,*] eq 1, nq )
	if ( nq GT 0 ) then fc[k] = median( foot[k,q], /even )
    endfor
    fc = edgemedian( fc, box )		; smooth one time

    df = fb - fc			    ; diff between scan and map

; perform cubic spline over the complete scan
    weight = double( flag[indx] EQ 0 )
    knots = nbb / ksep + 2
    if ( total( weight ) EQ 0 ) then return, !SUCCEED

    wgt = weight
    yfit = fit_cubic_splines( ind0, df, knots, weight=wgt, $
			    robust=rob, logpr=logpr, knotsep=ksep )

    if ( keyword_set( plot ) ) THEN BEGIN
	oplot, indx, fc, color=3
	oplot, indx, wgt * !y.crange[1] * 0.05, color=4
	plot, indx, df, position=[0.1,0.1,0.9,0.3], /noerase, ps=1
	oplot, indx, yfit, color=3
    endif

; investigate the glitch locations in order from begin to end
    g = where( flag[indx] GE 2, ng )
    if ( ng GT 0 ) THEN bg = append( g, nbb ) else bg = nbb
    bs = 0
    for k = 0, ng-1 do begin
	be = bg[k] - 1
	if ( be LT bs ) then continue	    ; starting glitch or double glitch

	wgt = weight[bs:be]
	bw = where( wgt GT 0, ln )
	if ( ln GT 4 ) then begin
	    knots = ln / ksep + 2
	    yfit1 = fit_cubic_splines( ind0[bs:be], df[bs:be], knots, $
			    weight=wgt, robust=rob, logpr=pr1, knotsep=ksep )

;	    oplot, indx[bs:be], yfit1, color=2
	endif else continue		; cannot determine -> cannot be better

	b2 = be + 2
	be = nbb-1
	if ( b2 GE be ) then break	    ; last point is glitch
	wgt = weight[b2:be]
	bw = where( wgt GT 0, ln )
	if ( ln GT 4 ) then begin
	    knots = ln / ksep + 2
	    yfit2 = fit_cubic_splines( ind0[b2:be], df[b2:be], knots, $
			    weight=wgt, robust=rob, logpr=pr2, knotsep=ksep )
;	    oplot, indx[b2:be], yfit2, color=2
	endif else continue		; cannot determine -> cannot be better

;	print, k, bs, b2-1, be, logpr, pr1, pr2, format='(4i6,3f10.4)'

	if ( pr1 + pr2 GT logpr ) then begin	    ; it is better => keep
	    logpr = pr2
	    yfit[bs:b2-2] = yfit1
	    yfit[b2:be]   = yfit2
	    bs = b2
	endif

    endfor

    if ( keyword_set( plot ) ) then begin
	oplot, indx, yfit, color=2, line=2
	waitforcursor, manual			; interactive plotting
    endif

    flux[indx] -= yfit					    ; apply correction

    return, !SUCCEED

END



