;+
; NAME:
;   ss_multi_scan_map2
; 
; PIRPOSE:
;   Make wide/deep image map with combining the multi-scan data
;
; MODIFICATION HISTORY:
;   Written by M.Shirahata, 2007/01/30
;   2007/02/02  M.Shirahata, fixed bug
;   2007/02/07  M.Shirahata, fixed bug
;   2007/02/08  S.Matsuura, added grid size input option.
;-
pro ss_multi_scan_map2, targetdir, SIGMA=sigma, T_START=t_start, T_END=t_end, $
			GRID_SW = grid_sw, GRID_LW = grid_lw

; Display the usage
IF (n_params() EQ 0) THEN BEGIN
    print, "# USAGE #"
    print, " ss_multi_scan, 'target_dir', sigma=value, t_start=value, t_end=value "
    print, "     memo.) sigma : threshold value of data rejection " 
    print, "            t_start & t_end : valied time range for making map, "
    print, "                              which are designated with the second [s] after 'ptm_mode' "
    RETURN
ENDIF


; Make an string array of target directory names
dirname_array = FILE_SEARCH(targetdir, /FOLD_CASE, /TEST_DIRECTORY)

; Show the total number of directories which were picked up
num_dir = N_Elements(dirname_array)
print, 'Search files in "'+targetdir+'" directories... '
print, Strcompress(String(num_dir)) + ' directories are found.'

; Loop for directories
FOR i=0, num_dir-1 DO BEGIN

    ; Find processed IDL files
    ; SW
    swfile = FILE_SEARCH(dirname_array[i]+'/FIS_SW*_ar.sav', /FOLD_CASE)
    title_w = 'Wide-S'
    title_n = 'N60'
    IF ((n_elements(swfile) EQ 1) && (swfile EQ '')) THEN BEGIN
        print, ' No SW file is found in the "'+dirname_array[i]+'" directory. '
    ENDIF ELSE BEGIN
    print, Strcompress(n_elements(swfile)) +' SW files are found.'
    ENDELSE
    ; LW
    lwfile = FILE_SEARCH(dirname_array[i]+'/FIS_LW*_ar.sav', /FOLD_CASE)
    title_w = 'Wide-L'
    title_n = 'N160'
    IF ((n_elements(lwfile) EQ 1) && (lwfile EQ '')) THEN BEGIN
        print, ' No LW file is found in "'+dirname_array[i]+'" directory. '
    ENDIF ELSE BEGIN
    print, Strcompress(n_elements(lwfile)) +' LW files are found.'
    ENDELSE

    ; SW Main start
    FOR j=0, n_elements(swfile)-1 DO BEGIN

        IF (strlen(swfile[j]) EQ 0) THEN BEGIN
            continue
        ENDIF

        ; Variables specified for SW detector
        c_pix = 20
        n_pix = 100
        w_pix = 60
        smpl_freq = 25.28D      ; [Hz]
        beam_size = 40.0D       ; FWHM in 1-D profile
        bad_flg = intarr(n_pix)
        bad_flg[0] = 1          ; ch1
        bad_flg[10] = 1         ; ch11
        bad_flg[70] = 1         ; ch71
        bad_flg[84] = 1         ; ch85
        bad_flg[97] = 1         ; ch98

        ; Read files
        print, 'Reading data file: ', swfile[j]
        restore, filename=swfile[j]
;        help, x, ra, dec, det, err, eg_gl_cnt, $
;          data_type, AOT, RST, rst_int, n_int, bad_flg

        ; Pick up data within valied time range
        ; Set the default value
        IF (n_elements(t_start) EQ 0) THEN BEGIN
            CASE AOT OF
                'PV_FIS01':       t_start=750
                'P12_FIS01':      t_start=630
                'P12_FIS01_SAA':  t_start=630
                'PV_FIS02':       t_start=750
                'P12_FIS02':      t_start=630
                'P12_FIS02_SAA':  t_start=630
            ENDCASE
        ENDIF
        IF (n_elements(t_end) EQ 0) THEN BEGIN
            t_end = 1350
        ENDIF
        print, 'Time range was set from', Strcompress(t_start) +' to', Strcompress(t_end) +'.'

        ; t_start
        t_start_idx = where(x gt t_start*smpl_freq, t_start_cnt)
        x = x[t_start_idx]
        n_line = n_elements(x)
        ra = ra[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        dec = dec[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        det = det[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        err = err[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        eg_gl_cnt = eg_gl_cnt[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        
        ; t_end
        t_end_idx = where(x lt t_end*smpl_freq, t_end_cnt)
        x = x[t_end_idx]
        n_line = n_elements(x)
        ra = ra[*,0:t_end_cnt-1]
        dec = dec[*,0:t_end_cnt-1]
        det = det[*,0:t_end_cnt-1]
        err = err[*,0:t_end_cnt-1]
        eg_gl_cnt = eg_gl_cnt[*,0:t_end_cnt-1]

        help, x, ra, dec, det, err, eg_gl_cnt, $
          data_type, AOT, RST, rst_int, n_int, bad_flg


        ; Read additional bad pixel table
        sw_basename = FILE_BASENAME(swfile[j], '_ar.sav')
        adbad_pix_file = dirname_array[i]+'/'+sw_basename+'_adbad.txt'
        OpenR, lun, adbad_pix_file, /Get_LUN, ERROR=oper
        IF(oper EQ 0) THEN BEGIN
            print, 'Reading additional bad pixel table from ', adbad_pix_file
            adbad_flg = intarr(n_pix)
            READF, lun, adbad_flg
            bad_flg = bad_flg + adbad_flg
            bad_pix = where(bad_flg gt 0, bad_pix_cnt)
            print, 'list of bad pixels: ', bad_pix + 1
            Free_lun,lun
        ENDIF ELSE BEGIN
            print, 'No additional bad pixel flag.'
        ENDELSE


        ; Make coadd data
        ; Data size check

        add_data_size = (size(x))[1]

        IF (n_elements(x_sw) EQ 0) THEN BEGIN
            x_sw = x
            ra_sw = ra
            dec_sw = dec
            det_sw = det
            err_sw = err
            eg_gl_cnt_sw = eg_gl_cnt
            data_type_sw = data_type
            AOT_sw = AOT
            RST_sw = RST
            rst_int_sw = rst_int
            n_int_sw = n_int
            bad_flg_sw = bad_flg
        ENDIF ELSE BEGIN

            IF ((data_type NE data_type_sw) || (AOT NE AOT_sw) || (RST NE RST_sw) || (n_int NE n_int_sw)) THEN BEGIN
                print, 'Data with a different observation mode (AOT & Reset) cannot be combined !'
                continue
            ENDIF ELSE BEGIN

                p_data_size = (size(x_sw))[1]
                n_data_size = p_data_size + add_data_size

                x_sw_p = x_sw
                ra_sw_p = ra_sw
                dec_sw_p = dec_sw
                det_sw_p = det_sw
                err_sw_p = err_sw
                eg_gl_cnt_sw_p = eg_gl_cnt_sw
            
                x_sw = dblarr(n_data_size)
                ra_sw = dblarr(100,n_data_size)
                dec_sw = dblarr(100,n_data_size)
                det_sw = dblarr(100,n_data_size)
                err_sw = dblarr(100,n_data_size)
                eg_gl_cnt_sw = dblarr(100,n_data_size)

                x_sw[0:p_data_size-1] = x_sw_p
                ra_sw[*,0:p_data_size-1] = ra_sw_p
                dec_sw[*,0:p_data_size-1] = dec_sw_p
                det_sw[*,0:p_data_size-1] = det_sw_p
                err_sw[*,0:p_data_size-1] = err_sw_p
                eg_gl_cnt_sw[*,0:p_data_size-1] = eg_gl_cnt_sw_p

                x_sw[p_data_size:n_data_size-1] = x
                ra_sw[*,p_data_size:n_data_size-1] = ra
                dec_sw[*,p_data_size:n_data_size-1] = dec
                det_sw[*,p_data_size:n_data_size-1] = det
                err_sw[*,p_data_size:n_data_size-1] = err
                eg_gl_cnt_sw[*,p_data_size:n_data_size-1] = eg_gl_cnt

            ENDELSE
        ENDELSE

    ; SW Loop end
    ENDFOR


    ; LW Main start
    FOR j=0, n_elements(lwfile)-1 DO BEGIN

        IF (strlen(lwfile[j]) EQ 0) THEN BEGIN
            continue
        ENDIF

        ; Variables specified for SW detector
        c_pix = 15
        n_pix = 75
        w_pix = 45
        smpl_freq = 16.86D      ; [Hz]
        beam_size = 70.0D       ; FWHM in 1-D profile
        bad_flg = intarr(n_pix)
        bad_flg[1] = 1          ; ch2
        bad_flg[2] = 1          ; ch3
        bad_flg[17] = 1         ; ch18
        bad_flg[32] = 1         ; ch33
        bad_flg[41] = 1         ; ch42
        bad_flg[47] = 1         ; ch48
        bad_flg[62] = 1         ; ch63
        bad_flg[69] = 1         ; ch70

        ; Read files
        print, 'Reading data file: ', lwfile[j]
        restore, filename=lwfile[j]
;        help, x, ra, dec, det, err, eg_gl_cnt, $
;          data_type, AOT, RST, rst_int, n_int, bad_flg

        ; Pick up data within valied time range
        ; Set the default value
        IF (n_elements(t_start) EQ 0) THEN BEGIN
            CASE AOT OF
                'PV_FIS01':       t_start=750
                'P12_FIS01':      t_start=630
                'P12_FIS01_SAA':  t_start=630
                'PV_FIS02':       t_start=750
                'P12_FIS02':      t_start=630
                'P12_FIS02_SAA':  t_start=630
            ENDCASE
        ENDIF
        IF (n_elements(t_end) EQ 0) THEN BEGIN
            t_end = 1350
        ENDIF
        print, 'Time range was set from', Strcompress(t_start) +' to', Strcompress(t_end) +'.'

        ; t_start
        t_start_idx = where(x gt t_start*smpl_freq, t_start_cnt)
        x = x[t_start_idx]
        n_line = n_elements(x)
        ra = ra[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        dec = dec[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        det = det[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        err = err[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        eg_gl_cnt = eg_gl_cnt[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
        
        ; t_end
        t_end_idx = where(x lt t_end*smpl_freq, t_end_cnt)
        x = x[t_end_idx]
        n_line = n_elements(x)
        ra = ra[*,0:t_end_cnt-1]
        dec = dec[*,0:t_end_cnt-1]
        det = det[*,0:t_end_cnt-1]
        err = err[*,0:t_end_cnt-1]
        eg_gl_cnt = eg_gl_cnt[*,0:t_end_cnt-1]

        help, x, ra, dec, det, err, eg_gl_cnt, $
          data_type, AOT, RST, rst_int, n_int, bad_flg


        ; Read additional bad pixel table
        lw_basename = FILE_BASENAME(lwfile[j], '_ar.sav')
        adbad_pix_file = dirname_array[i]+'/'+lw_basename+'_adbad.txt'
        OpenR, lun, adbad_pix_file, /Get_LUN, ERROR=oper
        IF(oper EQ 0) THEN BEGIN
            print, 'Reading additional bad pixel table from ', adbad_pix_file
            adbad_flg = intarr(n_pix)
            READF, lun, adbad_flg
            bad_flg = bad_flg + adbad_flg
            bad_pix = where(bad_flg gt 0, bad_pix_cnt)
            print, 'list of bad pixels: ', bad_pix + 1
            Free_lun,lun
        ENDIF ELSE BEGIN
            print, 'No additional bad pixel flag.'
        ENDELSE


        ; Make coadd data
        ; Data size check

        add_data_size = (size(x))[1]

        IF (n_elements(x_lw) EQ 0) THEN BEGIN
            x_lw = x
            ra_lw = ra
            dec_lw = dec
            det_lw = det
            err_lw = err
            eg_gl_cnt_lw = eg_gl_cnt
            data_type_lw = data_type
            AOT_lw = AOT
            RST_lw = RST
            rst_int_lw = rst_int
            n_int_lw = n_int
            bad_flg_lw = bad_flg
        ENDIF ELSE BEGIN

            IF ((data_type NE data_type_lw) || (AOT NE AOT_lw) || (RST NE RST_lw) || (n_int NE n_int_lw)) THEN BEGIN
                print, 'Data with a different observation mode (AOT & Reset) cannot be combined !'
                continue
            ENDIF ELSE BEGIN

                p_data_size = (size(x_lw))[1]
                n_data_size = p_data_size + add_data_size

                x_lw_p = x_lw
                ra_lw_p = ra_lw
                dec_lw_p = dec_lw
                det_lw_p = det_lw
                err_lw_p = err_lw
                eg_gl_cnt_lw_p = eg_gl_cnt_lw
            
                x_lw = dblarr(n_data_size)
                ra_lw = dblarr(75,n_data_size)
                dec_lw = dblarr(75,n_data_size)
                det_lw = dblarr(75,n_data_size)
                err_lw = dblarr(75,n_data_size)
                eg_gl_cnt_lw = dblarr(75,n_data_size)

                x_lw[0:p_data_size-1] = x_lw_p
                ra_lw[*,0:p_data_size-1] = ra_lw_p
                dec_lw[*,0:p_data_size-1] = dec_lw_p
                det_lw[*,0:p_data_size-1] = det_lw_p
                err_lw[*,0:p_data_size-1] = err_lw_p
                eg_gl_cnt_lw[*,0:p_data_size-1] = eg_gl_cnt_lw_p

                x_lw[p_data_size:n_data_size-1] = x
                ra_lw[*,p_data_size:n_data_size-1] = ra
                dec_lw[*,p_data_size:n_data_size-1] = dec
                det_lw[*,p_data_size:n_data_size-1] = det
                err_lw[*,p_data_size:n_data_size-1] = err
                eg_gl_cnt_lw[*,p_data_size:n_data_size-1] = eg_gl_cnt

            ENDELSE
        ENDELSE

    ; LW Loop end
    ENDFOR


; Make coad image

; SW
IF (n_elements(x_sw) EQ 0) THEN BEGIN
    print, 'No SW file --> No multi-scan map for SW.'
ENDIF ELSE BEGIN

    ; Output file name
    sw_map_name = targetdir+'/multi_scan_map2_sw'

    ; Check
    print, 'Combined data information for SW: '
    help, x_sw, ra_sw, dec_sw, det_sw, err_sw, eg_gl_cnt_sw, $
      sw_map_name, data_type_sw, AOT_sw, RST_sw, rst_int_sw, n_int_sw, bad_flg_sw

    ; Go to ss_coad_image
    newmapping = 1
    ss_coad_image, x_sw, ra_sw, dec_sw, det_sw, err_sw, eg_gl_cnt_sw, $
      sw_map_name, data_type_sw, AOT_sw, RST_sw, rst_int_sw, n_int_sw, bad_flg_sw, $
      im_ra, im_dec, coad_det, coad_num, coad_err, $
      ra_min, ra_max, dec_min, dec_max, $
      SIGMA = sigma, NEWMAPPING = newmapping, GRID_SW = grid_sw
    print, 'Multi-scan map for SW was saved as ', $
      Strcompress(sw_map_name)+'_img.sav & ',Strcompress(sw_map_name)+'_img_[w|n].fits'

ENDELSE

; LW
IF (n_elements(x_lw) EQ 0) THEN BEGIN
    print, 'No LW file --> No multi-scan map for LW.'
ENDIF ELSE BEGIN

    ; Output file name
    lw_map_name = targetdir+'/multi_scan_map2_lw'

    ; Check
    print, 'Combined data information for LW: '    
    help, x_lw, ra_lw, dec_lw, det_lw, err_lw, eg_gl_cnt_lw, $
      lw_map_name, data_type_lw, AOT_lw, RST_lw, rst_int_lw, n_int_lw, bad_flg_lw

    ; Go to ss_coad_image
    ss_coad_image, x_lw, ra_lw, dec_lw, det_lw, err_lw, eg_gl_cnt_lw, $
      lw_map_name, data_type_lw, AOT_lw, RST_lw, rst_int_lw, n_int_lw, bad_flg_lw, $
      im_ra, im_dec, coad_det, coad_num, coad_err, $
      ra_min, ra_max, dec_min, dec_max, $
      SIGMA = sigma, NEWMAPPING = newmapping, GRID_LW = grid_lw
    print, 'Multi-scan map for LW was saved as ', $
      Strcompress(lw_map_name)+'_img.sav & ',Strcompress(lw_map_name)+'_img_[w|n].fits'

ENDELSE


; Directory Loop end
ENDFOR

; END OF PRO
end
