;+
; NAME:
;   SS_RESP_CORR
;
; PURPOSE:
;   Return absolute responsivity calculated from local flat and dark data
;   or from pre-measured sky-flat data.
;   Responsivity is given in unit of (V/s)/(MJy/sr)
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   Written by S. Matsuura, 2006/10/16
;   2006/10/18 I. Yamamura CCF I/F generalized
;   2006/11/15 S. Matsuura include AOT information, combine plural cal tables
;   2006/12/07 S. Matsuura added AOT selection of special pointing sequence near SAA
;   2006/12/26 S. Matsuura RST-dependent responsivity factor for SW,
;                 introduced an input parameter RST
;   2007/01/07 S. Matsuura modified RST-dependent SW CAL-responsivity factor
;   2007/01/16 S. Matsuura, Revised CTOF table for pre-measured sky flat data
;                           Revised SW CAL-responsivity factor
;                           Absolute cal with CAL lamp data for local_flat option
;   2007/01/17 I.Yamamura,  ?W_ctof.dat moved from CCF/data to slowscan/lib.
;   2007/01/17 M.Shirahata, added sky_bg for sky subtraction.
;   2007/01/18 S.Matsuura, fixed bug in error of sky_ratio for local flat option.
;   2007/01/18 S.Matsuura, included 'P12_FIS02_SAA' to the choice list of AOT
;   2007/01/21 M.Shirahata, add print out sentence for mean responsivity.
;   2007/01/21 S.Matsuura, simple mean -> weighted mean for all calculation
;   2007/01/23 M.Shirahata, change output file names (.dat --> .txt)
;   2007/01/23 S.Matsuura, included bad data flag found in the flat fielding process.
;   2007/01/25 S.Matsuura, fixed minor bug.
;   2007/01/29 Makiuti, S. changed format code of E12.5 into E13.5 to avoid error on Windows
;   2007/02/01 M.Shirahata, fixed small bug in PV_FIS01 mode.
;   2007/02/03 S.Matsuura, added new parameter of bad data rejection threshold.
;   2007/02/06 S.Matsuura, modified bad pixel determination method.
;   2007/02/07 S.Matsuura, minor bug fix.
;
;-
pro ss_resp_corr, file_header, data_type, AOT, RST, dark, sky_bg, responsivity, $
        bad_flg, bad_thrshld, LOCAL_FLAT = local_flat

; parameters depending on data type
    print, 'Data type: ', data_type

    if(data_type eq 'FIS_SW') then begin
        n_pix = 100 ; total pix number
        w_pix = 60  ; wide band
        smpl_freq = 25.28D; [Hz]
    endif else begin
        n_pix = 75  ; total pix number
        w_pix = 45  ; wide band
        smpl_freq = 16.86D; [Hz]
    endelse


; read dark data table
    
    CASE AOT OF
        'PV_FIS01'	: n_dark = 11
        'P12_FIS01'	: n_dark = 8
        'P12_FIS01_SAA'	: n_dark = 8
        'PV_FIS02'	: n_dark = 9
        'P12_FIS02'	: n_dark = 6
        'P12_FIS02_SAA'	: n_dark = 6
    ENDCASE

    rdata = dblarr(4, n_pix) ; in [V/s]
    dark_multi = dblarr(4, n_pix, n_dark) ; in [V/s]

    for i_dark = 0, n_dark-1 do begin
        dark_num = strcompress(string(i_dark), /remove_all)
        dark_file = file_header + '_dark_' + dark_num + '.txt'
        openr,lun, dark_file, /Get_LUN, ERROR=oper
        if(oper eq 0) then begin
            print, 'Read dark table: ', dark_file
        endif else begin
            print, 'No dark table !!!'
            return
        endelse

        READF, lun, rdata
        Free_lun,lun

        dark_multi[*,*,i_dark] = rdata
    endfor


; Making mean dark table

    dark = dblarr(4, n_pix) ; in [V/s]
    dark[0,*] = dark_multi[0,*,1]

    if (AOT eq 'PV_FIS01') then begin
	use_pix = where(bad_flg eq 0 and dark_multi[2,*,1] gt 0 $
			and dark_multi[2,*,7] gt 0 and dark_multi[2,*,8] gt 0 , use_pix_cnt)
	if (use_pix_cnt gt 0) then begin
        dark[1,use_pix] = (dark_multi[1,use_pix,1]/dark_multi[2,use_pix,1]^2 $
		+dark_multi[1,use_pix,7]/dark_multi[2,use_pix,7]^2 $
		+dark_multi[1,use_pix,8]/dark_multi[2,use_pix,8]^2) $
		/(1D/dark_multi[2,use_pix,1]^2+1D/dark_multi[2,use_pix,7]^2 $
		+1D/dark_multi[2,use_pix,8]^2)
        dark[2,use_pix] = sqrt(1D/(1D/dark_multi[2,use_pix,1]^2 $
		+1D/dark_multi[2,use_pix,7]^2 $
		+1D/dark_multi[2,use_pix,8]^2))
        dark[3,use_pix] = dark_multi[3,use_pix,1] $
		+dark_multi[3,use_pix,7]+dark_multi[3,use_pix,8]
	endif
    endif
    if (AOT eq 'P12_FIS01' or AOT eq 'P12_FIS01_SAA') then begin
	use_pix = where(bad_flg eq 0 and dark_multi[2,*,1] gt 0 $
			and dark_multi[2,*,6] gt 0, use_pix_cnt)
	if (use_pix_cnt gt 0) then begin
        dark[1,use_pix] = (dark_multi[1,use_pix,1]/dark_multi[2,use_pix,1]^2 $
		+dark_multi[1,use_pix,6]/dark_multi[2,use_pix,6]^2) $
		/(1D/dark_multi[2,use_pix,1]^2+1D/dark_multi[2,use_pix,6]^2)
        dark[2,use_pix] = sqrt(1D/(1D/dark_multi[2,use_pix,1]^2 $
		+1D/dark_multi[2,use_pix,6]^2))
        dark[3,use_pix] = dark_multi[3,use_pix,1]+dark_multi[3,use_pix,6]
	endif
    endif
    if (AOT eq 'PV_FIS02') then begin
	use_pix = where(bad_flg eq 0 and dark_multi[2,*,1] gt 0 $
			and dark_multi[2,*,5] gt 0 and dark_multi[2,*,6] gt 0 , use_pix_cnt)
	if (use_pix_cnt gt 0) then begin
         dark[1,use_pix] = (dark_multi[1,use_pix,1]/dark_multi[2,use_pix,1]^2 $
		+dark_multi[1,use_pix,5]/dark_multi[2,use_pix,5]^2 $
		+dark_multi[1,use_pix,6]/dark_multi[2,use_pix,6]^2) $
		/(1D/dark_multi[2,use_pix,1]^2+1D/dark_multi[2,use_pix,5]^2 $
		+1D/dark_multi[2,use_pix,6]^2)
        dark[2,use_pix] = sqrt(1D/(1D/dark_multi[2,use_pix,1]^2 $
		+1D/dark_multi[2,use_pix,5]^2+1D/dark_multi[2,use_pix,6]^2))
        dark[3,use_pix] = dark_multi[3,use_pix,1] $
		+dark_multi[3,use_pix,5]+dark_multi[3,use_pix,6]
	endif
    endif
    if (AOT eq 'P12_FIS02' or AOT eq 'P12_FIS02_SAA') then begin
	use_pix = where(bad_flg eq 0 and dark_multi[2,*,1] gt 0 $
			and dark_multi[2,*,4] gt 0, use_pix_cnt)
	if (use_pix_cnt gt 0) then begin
        dark[1,use_pix] = (dark_multi[1,use_pix,1]/dark_multi[2,use_pix,1]^2 $
		+dark_multi[1,use_pix,4]/dark_multi[2,use_pix,4]^2) $
            /(1D/dark_multi[2,use_pix,1]^2+1D/dark_multi[2,use_pix,4]^2)
        dark[2,use_pix] = sqrt(1D/(1D/dark_multi[2,use_pix,1]^2 $
		+1D/dark_multi[2,use_pix,4]^2))
        dark[3,use_pix] = dark_multi[3,use_pix,1]+dark_multi[3,use_pix,4]
	endif
    endif


; read blank sky (local flat) data table

    pix_res_file = file_header + '_flat.txt'
    openr,lun, pix_res_file, /Get_LUN, ERROR=oper
    if(oper eq 0) then begin
        print, 'Read flat table: ', pix_res_file
    endif else begin
        print, 'No flat table !!!'
        return
    endelse

    flat = dblarr(4, n_pix) ; in [V/s]
    READF, lun, flat

    Free_lun,lun


; read cal data table
    
    CASE AOT OF
        'PV_FIS01'	: n_cal = 7
        'P12_FIS01'	: n_cal = 7
        'P12_FIS01_SAA'	: n_cal = 7
        'PV_FIS02'	: n_cal = 5
        'P12_FIS02'	: n_cal = 5
        'P12_FIS02_SAA'	: n_cal = 5
    ENDCASE

    rdata = dblarr(4, n_pix) ; in [V/s]
    cal_multi = dblarr(4, n_pix, n_cal) ; in [V/s]

    for i_cal = 0, n_cal-1 do begin
        cal_num = strcompress(string(i_cal), /remove_all)
        cal_file = file_header + '_cal_' + cal_num + '.txt'
        openr,lun, cal_file, /Get_LUN, ERROR=oper
        if(oper eq 0) then begin
            print, 'Read cal-light table: ', cal_file
        endif else begin
            print, 'No cal-light table !!!'
            return
        endelse

        READF, lun, rdata
        Free_lun,lun

        cal_multi[*,*,i_cal] = rdata
    endfor


; Making mean cal table

    cal = dblarr(4, n_pix) ; in [V/s]
    cal[0,*] = cal_multi[0,*,1]

    if (AOT eq 'PV_FIS01' or AOT eq 'P12_FIS01' or AOT eq 'P12_FIS01_SAA') then begin
	use_pix = where(bad_flg eq 0 and cal_multi[2,*,1] gt 0 $
			and cal_multi[2,*,5] gt 0, use_pix_cnt)
	if (use_pix_cnt gt 0) then begin
        cal[1,use_pix] = (cal_multi[1,use_pix,1]/cal_multi[2,use_pix,1]^2 $
		+cal_multi[1,use_pix,5]/cal_multi[2,use_pix,5]^2) $
		/(1D/cal_multi[2,use_pix,1]^2+1D/cal_multi[2,use_pix,5]^2)
        cal[2,use_pix] = sqrt(1D/(1D/cal_multi[2,use_pix,1]^2+1D/cal_multi[2,use_pix,5]^2))
        cal[3,use_pix] = cal_multi[3,use_pix,1]+cal_multi[3,use_pix,5]
	endif
    endif
    if (AOT eq 'PV_FIS02' or AOT eq 'P12_FIS02' or AOT eq 'P12_FIS02_SAA') then begin
	use_pix = where(bad_flg eq 0 and cal_multi[2,*,1] gt 0 $
			and cal_multi[2,*,3] gt 0, use_pix_cnt)
	if (use_pix_cnt gt 0) then begin
        cal[1,use_pix] = (cal_multi[1,use_pix,1]/cal_multi[2,use_pix,1]^2 $
		+cal_multi[1,use_pix,3]/cal_multi[2,use_pix,3]^2) $
		/(1D/cal_multi[2,use_pix,1]^2+1D/cal_multi[2,use_pix,3]^2)
        cal[2,use_pix] = sqrt(1D/(1D/cal_multi[2,use_pix,1]^2+1D/cal_multi[2,use_pix,3]^2))
        cal[3,use_pix] = cal_multi[3,use_pix,1]+cal_multi[3,use_pix,3]
	endif
    endif


;;;;;;;; extra text output ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; write cal-dark table

    cal_dark = dblarr(2,n_pix)  
    cal_dark[0,*] = cal[1,*]-dark[1,*]
    cal_dark[1,*] = sqrt(cal[2,*]^2+dark[2,*]^2)

    outfile = file_header + '_cal_dark.txt'
    openw, unit, outfile, /get_lun

    for pix = 0, n_pix-1 do begin
        printf, unit, pix+1, cal_dark[0,pix], cal_dark[1,pix], $
          format = '(I3, 2X, E13.5, 2X, E13.5)'
    endfor

    free_lun, unit

; write flat-dark table

    flat_dark = dblarr(2,n_pix) 
    flat_dark[0,*] = flat[1,*]-dark[1,*]
    flat_dark[1,*] = sqrt(flat[2,*]^2+dark[2,*]^2)

    outfile = file_header + '_flat_dark.txt'
    openw, unit, outfile, /get_lun

    for pix = 0, n_pix-1 do begin
        printf, unit, pix+1, flat_dark[0,pix], flat_dark[1,pix], $
          format = '(I3, 2X, E13.5, 2X, E13.5)'
    endfor

    free_lun, unit

; write (cal-dark)/(flat-dark) table

    ctof = dblarr(n_pix,2)  
    use_pix = where(bad_flg eq 0 and flat_dark[0,*] gt 0 and cal_dark[0,*] gt 0, use_pix_cnt)
    if (use_pix_cnt gt 0) then begin
	ctof[use_pix,0] = cal_dark[0,use_pix]/flat_dark[0,use_pix]
	ctof[use_pix,1] = sqrt((cal_dark[1,use_pix]/cal_dark[0,use_pix])^2 $
            +(flat_dark[1,use_pix]/flat_dark[0,use_pix])^2)*ctof[use_pix,0]
    endif

    outfile = file_header + '_ctof.txt'
    openw, unit, outfile, /get_lun

    for pix = 0, n_pix-1 do begin
        printf, unit, pix+1, ctof[pix,0], ctof[pix,1], $
        format = '(I3, 2X, E13.5, 2X, E13.5)'
    endfor

    free_lun, unit

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; read CTOF table (cal-light signal to flat sky signal ratio)

        root = GETENV('FISDR_ROOT')
 
        if(data_type eq 'FIS_SW') then begin
                ctof_file = root+'/slowscan/lib/SW_ctof.dat'
        endif else begin
                ctof_file = root+'/slowscan/lib/LW_ctof.dat'
        endelse

    ctof = dblarr(n_pix, 2)
    ctof0 = dblarr(2, n_pix)
    openr,lun, ctof_file, /Get_LUN, ERROR=oper
    if(oper eq 0) then begin
        print, 'Read CTOF table: ', ctof_file
    endif else begin
        print, 'No CTOF table !!!'
        return
    endelse
    READF, lun, ctof0
    Free_lun,lun
    ctof[*,0] = ctof0[0,*]
    ctof[*,1] = ctof0[1,*]


; read Responsivity table (derived from pre-measured flat sky data)

        root = GETENV('FISDR_ROOT')
 
        if(data_type eq 'FIS_SW') then begin
                resp_file = root+'/slowscan/lib/SW_resp.dat'
        endif else begin
                resp_file = root+'/slowscan/lib/LW_resp.dat'
        endelse

    resp = dblarr(n_pix, 2)
    resp0 = dblarr(2, n_pix)
    openr,lun, resp_file, /Get_LUN, ERROR=oper
    if(oper eq 0) then begin
        print, 'Read Responsivity table: ', resp_file
    endif else begin
        print, 'No Responsivity table !!!'
        return
    endelse
    READF, lun, resp0
    Free_lun,lun
    resp[*,0] = resp0[0,*]
    resp[*,1] = resp0[1,*]


; RST-dependent CAL intensity factor for SW (0:w|1:n|2:w-er|3:n-er)

    if(data_type eq 'FIS_SW') then begin

        sw_cal_fact = dblarr(4)
        sw_cal_fact[0] = 1.0D ; initial value
        sw_cal_fact[1] = 1.0D ; initial value
        sw_cal_fact[2] = 0.0D ; initial value
        sw_cal_fact[3] = 0.0D ; initial value

        ; Cal factor revised on 07/01/16 by Matsuura
        if (RST eq 'CDS' or RST eq '05N') then begin ; DAC 80h
            sw_cal_fact[0] = 1.587466D
            sw_cal_fact[1] = 1.681379D
        endif
        if (RST eq '10N') then begin ; DAC 60h
            sw_cal_fact[0] = 1.000000D
            sw_cal_fact[1] = 1.000000D
        endif
        if (RST eq '20N') then begin ; DAC 30h
            sw_cal_fact[0] = 0.292577D
            sw_cal_fact[1] = 0.237034D
        endif
        if (RST eq '20D') then begin ; DAC 10h
            sw_cal_fact[0] = 0.02924515D
            sw_cal_fact[1] = 0.01209704D
        endif

        ctof[0:59,0] = ctof[0:59,0]*sw_cal_fact[0]
        ctof[60:99,0] = ctof[60:99,0]*sw_cal_fact[1]

    endif

; pre-measured brightness of the flat sky

    sky = dblarr(2,4)
    sky[0,0] = 8.0080D  ; WS in MJy/sr dominated by Zodi at beta20
    sky[0,1] = 13.038D  ; N60 in MJy/sr dominated by Zodi at beta20
    sky[0,2] = 6.3490D  ; WL in MJy/sr dominated by Cirrus+Zodi at beta10
    sky[0,3] = 6.3490D  ; N160 in MJy/sr dominated by Cirrus+Zodi at beta10
    sky[1,*] = 1.0000D  ; Uncertainty of absolute sky brightness


; measure the sky_bg

        sky_bg = dblarr(2,n_pix) ; in [V/s]
        if(data_type eq 'FIS_SW') then begin
            sky_bg[0,*] = flat[1,*]-dark[1,*]
            sky_bg[1,*] = sqrt(flat[2,*]^2+dark[2,*]^2)
        endif else begin ; in case of 'FIS_LW'
            sky_bg[0,*] = flat[1,*]
            sky_bg[1,*] = flat[2,*]
        endelse


; flat fielding process

if(keyword_set(local_flat) eq 1) then begin
;;;;; Calibration by local flat fielding method ;;;;;

    sky_sig = dblarr(2,n_pix)   ; in [V/s]
    if(data_type eq 'FIS_SW') then begin
        sky_sig[0,*] = flat[1,*]-dark[1,*]
        sky_sig[1,*] = sqrt(flat[2,*]^2+dark[2,*]^2)
    endif else begin
        sky_sig[0,*] = flat[1,*]
        sky_sig[1,*] = flat[2,*]
    endelse

    ; bad pixel rejection
    fine_pix = where(bad_flg eq 0 and resp[*,0] gt 0 and sky_sig[0,*] gt 0, fine_pix_cnt)
    fine_w_idx = where(fine_pix lt w_pix, fine_w_cnt)
    fine_n_idx = where(fine_pix ge w_pix, fine_n_cnt)
    resp_ratio = dblarr(2,n_pix)
    resp_ratio[0,fine_pix[fine_w_idx]] = (sky_sig[0,fine_pix[fine_w_idx]]/resp[fine_pix[fine_w_idx],0]) $
                /mean(sky_sig[0,fine_pix[fine_w_idx]]/resp[fine_pix[fine_w_idx],0],/double,/NaN)
    resp_ratio[0,fine_pix[fine_n_idx]] = (sky_sig[0,fine_pix[fine_n_idx]]/resp[fine_pix[fine_n_idx],0]) $
                /mean(sky_sig[0,fine_pix[fine_n_idx]]/resp[fine_pix[fine_n_idx],0],/double,/NaN)    
    bad_pix_add = where(resp_ratio[0,fine_pix] gt bad_thrshld $
			or 1D/resp_ratio[0,fine_pix] gt bad_thrshld, bad_pix_add_cnt)
    if (bad_pix_add_cnt gt 0) then begin
        bad_flg[fine_pix[bad_pix_add]] = 1
        print, 'Bad pixels detected in flat fielding: ', fine_pix[bad_pix_add]+1
    endif else begin
        print, 'No bad pixel detected in flat fielding.'
    endelse

    ; adjust of absolute scale
    sky_ratio = dblarr(2,n_pix) ; signal ratio btw obs sky and flat sky
    fine_pix = where(bad_flg eq 0 and cal_dark[0,*] gt 0 and sky_sig[0,*] gt 0, use_pix_cnt)
    if (fine_pix_cnt gt 0) then begin
    	sky_ratio[0,fine_pix] = sky_sig[0,fine_pix]/cal_dark[0,fine_pix]*ctof[fine_pix,0]
    	sky_ratio[1,fine_pix] = sky_ratio[0,fine_pix] $
			*sqrt( (cal_dark[1,fine_pix]/cal_dark[0,fine_pix])^2 $
			+ (sky_sig[1,fine_pix]/sky_sig[0,fine_pix])^2 )
    endif
    fine_w_idx = where(fine_pix lt w_pix, fine_w_cnt)
    fine_n_idx = where(fine_pix ge w_pix, fine_n_cnt)
    if(data_type eq 'FIS_SW') then begin
        sky[0,0] = sky[0,0]*mean(sky_ratio[0,fine_pix[fine_w_idx]],/double,/NaN)
        sky[1,0] = sky[0,0]*stddev(sky_ratio[0,fine_pix[fine_w_idx]],/double,/NaN) $
            /sqrt(fine_w_cnt-1D)/mean(sky_ratio[0,fine_pix[fine_w_idx]],/double,/NaN)
        sky[0,1] = sky[0,1]*mean(sky_ratio[0,fine_pix[fine_n_idx]],/double,/NaN)
        sky[1,1] = sky[0,1]*stddev(sky_ratio[0,fine_pix[fine_n_idx]],/double,/NaN) $
            /sqrt(fine_n_cnt-1D)/mean(sky_ratio[0,fine_pix[fine_n_idx]],/double,/NaN)
    endif else begin ; in case of 'FIS_LW'
        sky[0,2] = sky[0,2]*mean(sky_ratio[0,fine_pix[fine_w_idx]],/double,/NaN)
        sky[1,2] = sky[0,2]*stddev(sky_ratio[0,fine_pix[fine_w_idx]],/double,/NaN) $
            /sqrt(fine_w_cnt-1D)/mean(sky_ratio[0,fine_pix[fine_w_idx]],/double,/NaN)
        sky[0,3] = sky[0,3]*mean(sky_ratio[0,fine_pix[fine_n_idx]],/double,/NaN)
        sky[1,3] = sky[0,3]*stddev(sky_ratio[0,fine_pix[fine_n_idx]],/double,/NaN) $
            /sqrt(fine_n_cnt-1D)/mean(sky_ratio[0,fine_pix[fine_n_idx]],/double,/NaN)
    endelse

    print,'Sky brightness W-S  [MJy/sr]:    ', sky[0,0]
    print,'Sky brightness N60  [MJy/sr]:    ', sky[0,1]
    print,'Sky brightness W-L(incl. B-light)[MJy/sr]:   ', sky[0,2]
    print,'Sky brightness N160(incl. B-light)[MJy/sr]:  ', sky[0,3]

endif else begin

;;;;; Calibration based on pre-measured flat sky data (Zodi and Cirrus) ;;;;;

    fine_pix = where(bad_flg eq 0 and ctof[*,0] gt 0, fine_pix_cnt)
    sky_sig = dblarr(2,n_pix)   ; in [V/s]
    sky_sig[0,fine_pix] = cal_dark[0,fine_pix]/ctof[fine_pix,0]
    sky_sig[1,fine_pix] = cal_dark[1,fine_pix]/ctof[fine_pix,0]

    print,'Sky brightness W-S  [MJy/sr]:    ', sky[0,0]
    print,'Sky brightness N60  [MJy/sr]:    ', sky[0,1]
    print,'Sky brightness W-L  [MJy/sr]:    ', sky[0,2]
    print,'Sky brightness N160 [MJy/sr]:    ', sky[0,3]

endelse



; Responsivity calculation

    responsivity = dblarr(2,n_pix)  ; in (V/s)/(MJy/sr)
    if(data_type eq 'FIS_SW') then begin
        for pix = 0, n_pix-1 do begin
        if(pix le w_pix-1) then begin ; Wide-S
	    if (sky[0,0] gt 0) then begin
            responsivity[0,pix] = sky_sig[0,pix]/sky[0,0]
            responsivity[1,pix] = sky_sig[1,pix]/sky[0,0]
	    endif
        endif else begin              ; N60
	    if (sky[0,1] gt 0) then begin
            responsivity[0,pix] = sky_sig[0,pix]/sky[0,1]
            responsivity[1,pix] = sky_sig[1,pix]/sky[0,1]
	    endif
        endelse
        endfor
    endif else begin ; 'FIS_LW'
        for pix = 0, n_pix-1 do begin
        if(pix le w_pix-1) then begin ; Wide-L
	    if (sky[0,2] gt 0) then begin
            responsivity[0,pix] = sky_sig[0,pix]/sky[0,2]
            responsivity[1,pix] = sky_sig[1,pix]/sky[0,2]
	    endif
        endif else begin              ; N160
	    if (sky[0,3] gt 0) then begin
            responsivity[0,pix] = sky_sig[0,pix]/sky[0,3]
            responsivity[1,pix] = sky_sig[1,pix]/sky[0,3]
	    endif
        endelse
        endfor
    endelse

    ; bad pixel rejection
    fine_pix = where(bad_flg eq 0 and resp[*,0] gt 0 and responsivity[0,*] gt 0, fine_pix_cnt)
    fine_w_idx = where(fine_pix lt w_pix, fine_w_cnt)
    fine_n_idx = where(fine_pix ge w_pix, fine_n_cnt)
    resp_ratio2 = dblarr(2,n_pix)
    resp_ratio2[0,fine_pix[fine_w_idx]] = (responsivity[0,fine_pix[fine_w_idx]]/resp[fine_pix[fine_w_idx],0]) $
                /mean(responsivity[0,fine_pix[fine_w_idx]]/resp[fine_pix[fine_w_idx],0],/double,/nan)
    resp_ratio2[0,fine_pix[fine_n_idx]] = (responsivity[0,fine_pix[fine_n_idx]]/resp[fine_pix[fine_n_idx],0]) $
                /mean(responsivity[0,fine_pix[fine_n_idx]]/resp[fine_pix[fine_n_idx],0],/double,/nan)   
    bad_pix_add2 = where(resp_ratio2[0,fine_pix] gt bad_thrshld $
			or 1D/resp_ratio2[0,fine_pix] gt bad_thrshld, bad_pix_add2_cnt)
    if (bad_pix_add2_cnt gt 0) then begin
        bad_flg[fine_pix[bad_pix_add2]] = 1
        print, 'Bad pixels detected in responsivity check: ', fine_pix[bad_pix_add2]+1
        responsivity[fine_pix[bad_pix_add2]] = resp[fine_pix[bad_pix_add2],0]
        fine_pix = where(bad_flg eq 0, fine_pix_cnt)
        fine_w_idx = where(fine_pix lt w_pix, fine_w_cnt)
        fine_n_idx = where(fine_pix ge w_pix, fine_n_cnt)
    endif else begin
        print, 'No bad pixel detected in responsivity check.'
    endelse
    print, 'Number of healthy pixels (WIDE): ', fine_w_cnt
    print, 'Number of healthy pixels (NARROW): ', fine_n_cnt

    ; mean sky brightness
        if(data_type eq 'FIS_SW') then begin
            mean_resp_ws = 0.0D
            std_resp_ws = 0.0D
            mean_resp_ns = 0.0D
            cnt_ws = 0
            cnt_ns = 0
            for pix = 0, w_pix-1 do begin ; Wide-S
                if (responsivity[0,pix] gt 0 and bad_flg[pix] ne 1) then begin
                    cnt_ws = cnt_ws + 1
                    mean_resp_ws = mean_resp_ws + responsivity[0,pix]
                endif
            endfor
            for pix = w_pix, n_pix-1 do begin ; N60
                if (responsivity[0,pix] gt 0 and bad_flg[pix] ne 1) then begin
                    cnt_ns = cnt_ns + 1
                    mean_resp_ns = mean_resp_ns + responsivity[0,pix]
                endif
            endfor
            mean_resp_ws = mean_resp_ws / cnt_ws
            mean_resp_ns = mean_resp_ns / cnt_ns
            print, 'Mean responsivity W-S [(V/s)/(MJy/sr)]: ', mean_resp_ws
            print, 'Mean responsivity N60 [(V/s)/(MJy/sr)]: ', mean_resp_ns
        endif
        if(data_type eq 'FIS_LW') then begin
            mean_resp_wl = 0.0D
            mean_resp_nl = 0.0D
            cnt_wl = 0
            cnt_nl = 0
            for pix = 0, w_pix-1 do begin ; Wide-L
                if (responsivity[0,pix] gt 0 and bad_flg[pix] ne 1) then begin
                    cnt_wl = cnt_wl + 1
                    mean_resp_wl = mean_resp_wl + responsivity[0,pix]
                endif
            endfor
            for pix = w_pix, n_pix-1 do begin ; N160
                if (responsivity[0,pix] gt 0 and bad_flg[pix] ne 1) then begin
                    cnt_nl = cnt_nl + 1
                    mean_resp_nl = mean_resp_nl + responsivity[0,pix]
                endif
            endfor
            mean_resp_wl = mean_resp_wl / cnt_wl
            mean_resp_nl = mean_resp_nl / cnt_nl
            print, 'Mean responsivity W-L [(V/s)/(MJy/sr)]: ', mean_resp_wl
            print, 'Mean responsivity N160 [(V/s)/(MJy/sr)]:    ', mean_resp_nl
        endif

END
