;----------------------------------------------------------------------------
;+
;
; NAME:
;   Get_INFOARR_From_LDS
;
; PURPOSE:
;   Query and get an array of TSD objects infomation from LDS
;
; CALLING SEQUENCE:
;   infoarr = Get_INFOARR_From_LDS(MODE=mode, TIME=time, SPAN=span, $
;                RA=ra, DEC=dec,SIZE=size, $
;                [/J2000,/B1950],[/EQUATORIAL,/GALACTIC,/ECLIPTIC],$
;                FORMAT=format, INDEX=index, GB_PROCESSED=gb, /COMPRESS, /KEEP)
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   MODE	Data query mode: ("Lab" or "Sim" or "Fli": default: "Lab")
;   TIME	Start time of request data
;		Time should be JST for Lab data, and UTC for Sim data.
;               (STRING FORMAT: '2004-07-10 23:40:00')
;   SPAN	Span of request data (seconds)
;   RA		Coordinates (degree)
;   DEC		Coordinates (degree)
;   SIZE        Size of area (degree)
;   /J2000      Original equinox of coordinates (default)
;   /B1950      Another equinox of coordinates
;   /EQUATORIAL Input is equatorial coordinate system(RA,DEC) (default)
;   /GALACTIC   Input is galactic coordinate system
;   /ECLIPTIC   Input is ecliptic coordinate system
;   FORMAT	Output Format
;		(now available: 'DAT','TSD4_LW','TSD4_SW',
;		 'TSD3_LW','TSD3_SW','TSD2_LW','TSD2_SW')
;   INDEX       Index array which specifies row numbers to be downloaded.
;               (default: not set -> all data should be downloaded.)
;   GB_PROCESSED Flag whether data is processed by GreenBox or not
;               (0: not_processed, 1: processed.  Default: 0)
;   /COMPRESS	Get a compressed (gzipped) data or not (default: no)
;   /KEEP	Keep temporarily downloaded file if set.
;		(note that on-memory object is not removed.)
;
; OUTPUTS:
;   infoarr	Array of information structure
;		Return !FAIL if fail.
;
; OUTPUT KEYWORDS:
;   NONE
;
; RESTRICTIONS:
;   Return values are undefined if both time and coordinates are specified.
;
; COMMENTS:
;   * Availability of the Internet connection.  In detail, be able to
;     access to the LDS server '!LDS_URL' via port 80 (HTTP).
;   * This method is for non-interactive use.  You can access to LDS
;     interactively via: '!LDS_URL'.
;   * This function should not run in IDL demo mode.
; 
; EXAMPLE:
;     IDL> infoarr = Get_INFOARR_From_LDS(MODE='lab', TIME='2004-07-10 23:40:00', SPAN=10, FORMAT='TSD4_LW')
;     IDL> print, infoarr[0].filename
;     AFTIME          INT       = Array[473]
;
;-
;----------------------------------------------------------------------------

FUNCTION Get_INFOARR_From_LDS, MODE=mode, TIME=time, SPAN=span, $
                              RA=ra, DEC=dec, SIZE=size, $
                              B1950=b1950, GALACTIC=galactic, $
                              FORMAT=format, $
                              INDEX=index, GB_PROCESSED=gb, $
                              COMPRESS=compress, KEEP=keep
    COMPILE_OPT hidden
    ;;;;
    RESOLVE_ROUTINE, 'util'
    util
    setDebug
    ;;;;
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    IF (NOT (KEYWORD_SET(time) AND KEYWORD_SET(span)) AND $
        NOT (KEYWORD_SET(ra) AND KEYWORD_SET(dec) AND KEYWORD_SET(size))) $
      THEN BEGIN
        MESSAGE, "Keyword sets ('TIME','SPAN') or ('RA','DEC','SIZE') should be set.", /CONTINUE
        MESSAGE, "NOTE: Return values are undefined if both time and coordinates are specified.", /CONTINUE
        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF

    IF KEYWORD_SET(ra) AND KEYWORD_SET(dec) THEN BEGIN
        ; default: (ra,dec) in J2000
        ra2000 = ra
        de2000 = dec
        IF KEYWORD_SET(b1950) THEN BEGIN
            ; astrolib
            JPRECESS, ra, dec, ra2000, de2000
        ENDIF
        IF KEYWORD_SET(galactic) THEN BEGIN
            ; j=2: convert gl,gb  --> ra,dec
            GLACTC, ra2000, de2000, 2000, ra, dec, 2
        ENDIF
    ENDIF ELSE BEGIN
        ra2000 = -999
        de2000 = -999
        size = 0
    ENDELSE

    IF NOT KEYWORD_SET(mode) THEN mode = "Lab"
    available_modes = ["LAB", "SIM", "FLI"]
    tmp = WHERE(STRMATCH(available_modes, STRUPCASE(mode), /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        MESSAGE, 'No such mode: ' + mode, /CONTINUE
        MESSAGE, 'Available modes are: ' + $
                 STRJOIN(available_modes,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(format) THEN format = "DAT"
    available_formats = ['DAT', 'TSD4_LW', 'TSD4_SW', 'TSD3_LW', 'TSD3_SW', 'TSD2_LW', 'TSD2_SW']
    tmp = WHERE(STRMATCH(available_formats, STRUPCASE(format), /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        MESSAGE, 'No such format: ' + format, /CONTINUE
        MESSAGE, 'Available formats are: ' + $
                 STRJOIN(available_formats,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(index) AND N_ELEMENTS(index) EQ 0 THEN index = [-1]
    IF SIZE(index, /DIMENSIONS) EQ 0 THEN index = [index] ; scalar -> array
    IF NOT KEYWORD_SET(gb) THEN gb = 0
    IF NOT KEYWORD_SET(compress) THEN compress = 0
    IF NOT KEYWORD_SET(keep) THEN keep = 0

    DPRINT, 'LDS_URL: ' + !LDS_URL
    DPRINT, 'Query TSD from LDS with following condition:'
    DPRINT, 'mode="' + mode + '", time="' + time + '", span="' + STRTRIM(STRING(span),2) + '", format="' + format + '"'

    ; Download TSD files from LDS via XMLRPC interface
    xmlrpc = OBJ_NEW('xmlrpc')
    proxy = !LDS_URL + 'xmlrpc.cgi'
    method = 'LDS.queryViaXMLRPC'
;    proxy = !LDS_URL + 'xmlrpc_payload.cgi'
;    method = 'LDS.queryViaXMLRPC_Mini_Payload'
    params = { PARAMS, mode: mode, time: time, span: span, $
               ra2000: ra2000, de2000: de2000, size: size, $
               format: format, index: index, gb_processed: gb, gzip: compress }
    xmlrpc = xmlrpc->proxy(proxy)
    xmlrpc = xmlrpc->call(method, params, KEEP=keep)
    result = xmlrpc->result()
    IF xmlrpc->isFault() EQ 1 THEN BEGIN
        MESSAGE, 'XMLRPC ERROR: ', /CONTINUE
        MESSAGE, 'faultCode:   ' + STRING(result.faultCode), /CONTINUE
        MESSAGE, 'faultString: ' + result.faultString, /CONTINUE
        OBJ_DESTROY, xmlrpc
        ; STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    OBJ_DESTROY, xmlrpc

    num = N_TAGS(result.param_0)
    IF num EQ 0 THEN BEGIN
        MESSAGE, 'No available data.', /CONTINUE
        RETURN, !FAIL
    ENDIF

    RETURN, result.param_0
END

