;----------------------------------------------------------------------------
;+
;
; NAME:
;   Get_TERMINFOARR_From_LDS
;
; PURPOSE:
;   Query and get an array of TSD objects infomation from LDS
;
; CALLING SEQUENCE:
;   infoarr = Get_INFOARR_From_LDS(MODE=mode, TIME=time, SPAN=span, $
;                RA=ra, DEC=dec,SIZE=size, $
;                [/J2000,/B1950],[/EQUATORIAL,/GALACTIC,/ECLIPTIC],$
;                FORMAT=format, INDEX=index, GB_PROCESSED=gb, /COMPRESS, /KEEP)
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   MODE	Data query mode: ("Lab" or "Sim" or "Fli": default: "Lab")
;   TIME	Start time of request data
;		Time should be JST for Lab data, and UTC for Sim data.
;               (STRING FORMAT: '2004-07-10 23:40:00')
;   SPAN	Span of request data (seconds)
;   RA		Coordinates (degree)
;   DEC		Coordinates (degree)
;   SIZE        Size of area (degree)
;   /J2000      Original equinox of coordinates (default)
;   /B1950      Another equinox of coordinates
;   /EQUATORIAL Input is equatorial coordinate system(RA,DEC) (default)
;   /GALACTIC   Input is galactic coordinate system
;   /ECLIPTIC   Input is ecliptic coordinate system
;   FORMAT	Output Format
;		(now available: 'DAT','TSD4_LW','TSD4_SW',
;		 'TSD3_LW','TSD3_SW','TSD2_LW','TSD2_SW')
;   INDEX       Index array which specifies row numbers to be downloaded.
;               (default: not set -> all data should be downloaded.)
;   GB_PROCESSED Flag whether data is processed by GreenBox or not
;               (0: not_processed, 1: processed.  Default: 0)
;   /COMPRESS	Get a compressed (gzipped) data or not (default: no)
;   /KEEP	Keep temporarily downloaded file if set.
;		(note that on-memory object is not removed.)
;
; OUTPUTS:
;   infoarr	Array of information structure
;		Return !FAIL if fail.
;
; OUTPUT KEYWORDS:
;   NONE
;
; RESTRICTIONS:
;   Return values are undefined if both time and coordinates are specified.
;
; COMMENTS:
;   * Availability of the Internet connection.  In detail, be able to
;     access to the LDS server '!LDS_URL' via port 80 (HTTP).
;   * This method is for non-interactive use.  You can access to LDS
;     interactively via: '!LDS_URL'.
;   * This function should not run in IDL demo mode.
; 
; EXAMPLE:
;     IDL> infoarr = Get_INFOARR_From_LDS(MODE='lab', TIME='2004-07-10 23:40:00', SPAN=10, FORMAT='TSD4_LW')
;     IDL> print, infoarr[0].filename
;     AFTIME          INT       = Array[473]
;
;-
;----------------------------------------------------------------------------

FUNCTION Get_TERMINFOARR_From_LDS, MODE=mode, TIME=time, SPAN=span, $
                                   FORMAT=format, GB_PROCESSED=gb
    COMPILE_OPT hidden
    ;;;;
    RESOLVE_ROUTINE, 'util'
    util
    setDebug
    ;;;;
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    IF NOT (KEYWORD_SET(time) AND KEYWORD_SET(span)) THEN BEGIN
        MESSAGE, "Keyword sets ('TIME','SPAN') should be set.", /CONTINUE
        MESSAGE, "NOTE: Return values are undefined if both time and coordinates are specified.", /CONTINUE
        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(mode) THEN mode = "Lab"
    available_modes = ["LAB", "SIM", "FLI"]
    tmp = WHERE(STRMATCH(available_modes, STRUPCASE(mode), /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        MESSAGE, 'No such mode: ' + mode, /CONTINUE
        MESSAGE, 'Available modes are: ' + $
                 STRJOIN(available_modes,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(format) THEN format = "DAT"
    available_formats = ['DAT', 'TSD4_LW', 'TSD4_SW', 'TSD3_LW', 'TSD3_SW', 'TSD2_LW', 'TSD2_SW']
    tmp = WHERE(STRMATCH(available_formats, STRUPCASE(format), /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        MESSAGE, 'No such format: ' + format, /CONTINUE
        MESSAGE, 'Available formats are: ' + $
                 STRJOIN(available_formats,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(gb) THEN gb = 0

    DPRINT, 'LDS_URL: ' + !LDS_URL
    DPRINT, 'Query TSD from LDS with following condition:'
    ;DPRINT, 'mode="' + mode + '", time="' + time + '", span="' + STRTRIM(STRING(span),2) + '", format="' + format + '"'

    ; Download TSD files from LDS via XMLRPC interface
    xmlrpc = OBJ_NEW('xmlrpc')
    ;proxy = !LDS_URL + 'xmlrpc.cgi'
    proxy = !LDS_URL + 'nph-xmlrpc_oreload.cgi'
    method = 'LDS.termqueryViaXMLRPC_Oreload'
;    proxy = !LDS_URL + 'xmlrpc_payload.cgi'
;    method = 'LDS.queryViaXMLRPC_Mini_Payload'

    ra2000 = -999
    de2000 = -999
    size = 0
    compress = 0
    params = { PARAMS, mode: mode, time: time, span: span, $
               ra2000: ra2000, de2000: de2000, size: size, $
               format: format, gb_processed: gb, gzip: compress }
    xmlrpc = xmlrpc->proxy(proxy)
    xmlrpc = xmlrpc->call(method, params, KEEP=keep)
    result = xmlrpc->result()
    IF xmlrpc->isFault() EQ 1 THEN BEGIN
        MESSAGE, 'XMLRPC ERROR: ', /CONTINUE
        MESSAGE, 'faultCode:   ' + STRING(result.faultCode), /CONTINUE
        MESSAGE, 'faultString: ' + result.faultString, /CONTINUE
        OBJ_DESTROY, xmlrpc
        ; STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    OBJ_DESTROY, xmlrpc

    num = N_TAGS(result.param_0)
    IF num EQ 0 THEN BEGIN
        MESSAGE, 'No available data.', /CONTINUE
        RETURN, !FAIL
    ENDIF

    RETURN, result.param_0
END


