;----------------------------------------------------------------------------
;+
;
; NAME:
;   Get_TestDataSet_From_LDS
;
; PURPOSE:
;   Query and get an array of TSD objects from LDS (useful for IT#3)
;
; CALLING SEQUENCE:
;   tsdarr = Get_TestDataSet_From_LDS(DATASETID=datasetid, $
;                                     INDEX=index, GB_PROCESSED=gb, $
;                                     /INFOONLY, /COMPRESS, /KEEP)
;
; INPUTS:
;   DATASETID   DatasetID
;
; OPTIONAL KEYWORDS:
;   INDEX       Index array which specifies row numbers to be downloaded.
;               (default: not set -> all data should be downloaded.)
;   GB_PROCESSED Flag whether data is processed by GreenBox or not
;               (0: not_processed, 1: processed.  Default: 0)
;   /INFOONLY   Download information only (not data) if set.
;               (0: download tsd data, 1: infomation only.  default: 0)
;   /COMPRESS	Get a compressed (gzipped) data or not (default: no)
;   /KEEP	Keep temporarily downloaded file if set.
;		(note that on-memory object is not removed.)
;
; OUTPUTS:
;   tsdarr	An array of TSD objects
;		Return !FAIL if fail.
;
; OUTPUT KEYWORDS:
;   NONE
;
; COMMENTS:
;   * Availability of the Internet connection.  In detail, be able to
;     access to the LDS server '!LDS_URL' via port 80 (HTTP).
;   * This method is for non-interactive use.  You can access to LDS
;     interactively via: !LDS_URL
;   * This function should not run in IDL demo mode.
; 
; EXAMPLE:
;     IDL> tsdarr = Get_TestDataSet_From_LDS(DATASETID='setee06_ss501_sw_gb001_p2p_sglitch')
;     IDL> aftime = tsdarr[0]->Get_Value_TSD('FIS_OBS', 'AFTIME')
;     IDL> help, aftime
;     AFTIME          INT       = Array[473]
;
;-
;----------------------------------------------------------------------------

FUNCTION Get_TestDataSet_From_LDS, DATASETID=datasetid, $
                                   GB_PROCESSED=gb, $
                                   INDEX=index, $
                                   INFOONLY=infoonly, $
                                   COMPRESS=compress, KEEP=keep
    COMPILE_OPT hidden
    ;;;;
    RESOLVE_ROUTINE, 'util'
    util
    setDebug
    ;;;;
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    ; list of data-set IDs
    datasetids = [ $
                   'setee10_ss501_sw_gb002_crs_sglitch',$
                   'setee10_ss502_lw_gb002_crs_sglitch',$
                   'setee10_ss501_sw_gb002_crs',$
                   'setee10_ss502_lw_gb002_crs',$
                   'setee10_ss501_sw_gb002_sglitch',$
                   'setee10_ss502_lw_gb002_sglitch',$
                   'setee10_ss501_sw_gb002',$
                   'setee10_ss502_lw_gb002',$
                   'setee07_ss501_sw_gb001_tr_sglitch',$
                   'setee07_ss502_lw_gb001_tr_sglitch',$
                   'setee07_ss501_sw_gb001_tr',$
                   'setee07_ss502_lw_gb001_tr',$
                   'setee07_ss501_sw_gb001_sglitch',$
                   'setee07_ss502_lw_gb001_sglitch',$
                   'setee07_ss501_sw_gb001',$
                   'setee07_ss502_lw_gb001',$
                   'setee07_ss501_sw_gb002_tr_sglitch',$
                   'setee07_ss502_lw_gb002_tr_sglitch',$
                   'setee07_ss501_sw_gb002_tr',$
                   'setee07_ss502_lw_gb002_tr',$
                   'setee07_ss501_sw_gb002_sglitch',$
                   'setee07_ss502_lw_gb002_sglitch',$
                   'setee07_ss501_sw_gb002',$
                   'setee07_ss502_lw_gb002',$
                   'setee06_ss501_sw_gb001_p2p_sglitch',$
                   'setee06_ss502_lw_gb001_p2p_sglitch',$
                   'setee06_ss501_sw_gb001_p2p',$
                   'setee06_ss502_lw_gb001_p2p',$
                   'setee06_ss501_sw_gb001_sglitch',$
                   'setee06_ss502_lw_gb001_sglitch',$
                   'setee06_ss501_sw_gb001',$
                   'setee06_ss502_lw_gb001',$
                   'setee06_ss501_sw_gb002_p2p_sglitch',$
                   'setee06_ss502_lw_gb002_p2p_sglitch',$
                   'setee06_ss501_sw_gb002_p2p',$
                   'setee06_ss502_lw_gb002_p2p',$
                   'setee06_ss501_sw_gb002_sglitch',$
                   'setee06_ss502_lw_gb002_sglitch',$
                   'setee06_ss501_sw_gb002',$
                   'setee06_ss502_lw_gb002',$
                   'setee05_ss506_sw_gb002_sglitch', $
                   'setee05_ss506_lw_gb002_sglitch', $
                   'setee05_ss506_sw_gb002', $
                   'setee05_ss506_lw_gb002', $
                   'setee05_ss505_sw_gb002_sglitch', $
                   'setee05_ss505_lw_gb002_sglitch', $
                   'setee05_ss505_sw_gb002', $
                   'setee05_ss505_lw_gb002', $
                   'setee04_ss500_sw_gb002',$
                   'setee04_ss500_lw_gb002',$
                   'setee03_ss501_sw_gb001_sglitch',$
                   'setee03_ss502_lw_gb001_sglitch',$
                   'setee03_ss501_sw_gb001',$
                   'setee03_ss502_lw_gb001',$
                   'setee03_ss501_sw_gb002_sglitch',$
                   'setee03_ss502_lw_gb002_sglitch',$
                   'setee03_ss501_sw_gb002',$
                   'setee03_ss502_lw_gb002',$
                   'setee02_ss501_sw_gb002_sglitch',$
                   'setee02_ss502_lw_gb002_sglitch',$
                   'setee02_ss501_sw_gb002',$
                   'setee02_ss502_lw_gb002',$
                   'setee01_ss501_sw_gb002',$
                   'setee01_ss502_lw_gb002',$
                   'setee01_ss500_sw_gb002',$
                   'setee01_ss500_lw_gb002',$
                   'setee01_ss499_sw_gb002',$
                   'setee01_ss499_lw_gb002',$
                   'set17_sw',$
                   'set17_lw',$
                   'set12_sw',$
                   'set12_lw',$
                   'set1b_lw',$
                   'set1b_sw',$
                   'set1a_lw',$
                   'set1a_sw' ]

    IF NOT KEYWORD_SET(datasetid) THEN BEGIN
        LMESSAGE, 0, "Keyword 'DATASETID' should be set.", /CONTINUE
        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF

    tmp = WHERE(STRMATCH(datasetids, datasetid, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        LMESSAGE, 0, 'No such DataSetID: "' + datasetid + '"', /CONTINUE
        LMESSAGE, 3, 'The available IDs are: ', /CONTINUE, /NONAME
        LMESSAGE, 3, STRJOIN(datasetids,', '), /CONTINUE, /NONAME
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    CASE STRLOWCASE(datasetid) OF
        ;; ee10
        'setee10_ss501_sw_gb002_crs_sglitch': BEGIN
            time='2006-08-20 00:00:10'
            span=211000 ; 2006-08-22 10:22:49
            format = "TSD3_SW"
        END
        'setee10_ss502_lw_gb002_crs_sglitch': BEGIN
            time='2006-08-20 00:00:10'
            span=211000 ; 2006-08-22 10:22:49
            format = "TSD3_LW"
        END
        'setee10_ss501_sw_gb002_crs': BEGIN
            time='2006-08-15 00:00:10'
            span=211000 ; 2006-08-17 10:22:49
            format = "TSD3_SW"
        END
        'setee10_ss502_lw_gb002_crs': BEGIN
            time='2006-08-15 00:00:10'
            span=211000 ; 2006-08-17 10:22:49
            format = "TSD3_LW"
        END
        'setee10_ss501_sw_gb002_sglitch': BEGIN
            time='2006-08-06 00:00:10'
            span=211000 ; 2006-08-08 10:22:49
            format = "TSD3_SW"
        END
        'setee10_ss502_lw_gb002_sglitch': BEGIN
            time='2006-08-06 00:00:10'
            span=211000 ; 2006-08-08 10:22:49
            format = "TSD3_LW"
        END
        'setee10_ss501_sw_gb002': BEGIN
            time='2006-08-01 00:00:10'
            span=211000 ; 2006-08-03 10:22:49
            format = "TSD3_SW"
        END
        'setee10_ss502_lw_gb002': BEGIN
            time='2006-08-01 00:00:10'
            span=211000 ; 2006-08-03 10:22:49
            format = "TSD3_LW"
        END


        ;; ee07
        'setee07_ss501_sw_gb001_tr_sglitch': BEGIN
            time='2006-07-15 00:00:10'
            span=211000 ; 2006-07-17 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb001_tr_sglitch': BEGIN
            time='2006-07-15 00:00:10'
            span=211000 ; 2006-07-17 10:22:49
            format = "TSD3_LW"
        END
        'setee07_ss501_sw_gb001_tr': BEGIN
            time='2006-07-10 00:00:10'
            span=211000 ; 2006-07-12 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb001_tr': BEGIN
            time='2006-07-10 00:00:10'
            span=211000 ; 2006-07-12 10:22:49
            format = "TSD3_LW"
        END
        'setee07_ss501_sw_gb001_sglitch': BEGIN
            time='2006-07-05 00:00:10'
            span=211000 ; 2006-07-07 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb001_sglitch': BEGIN
            time='2006-07-05 00:00:10'
            span=211000 ; 2006-07-07 10:22:49
            format = "TSD3_LW"
        END
        'setee07_ss501_sw_gb001': BEGIN
            time='2006-06-30 00:00:10'
            span=211000 ; 2006-07-02 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb001': BEGIN
            time='2006-06-30 00:00:10'
            span=211000 ; 2006-07-02 10:22:49
            format = "TSD3_LW"
        END
        'setee07_ss501_sw_gb002_tr_sglitch': BEGIN
            time='2006-06-25 00:00:10'
            span=211000 ; 2006-06-27 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb002_tr_sglitch': BEGIN
            time='2006-06-25 00:00:10'
            span=211000 ; 2006-06-27 10:22:49
            format = "TSD3_LW"
        END
        'setee07_ss501_sw_gb002_tr': BEGIN
            time='2006-06-20 00:00:10'
            span=211000 ; 2006-06-22 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb002_tr': BEGIN
            time='2006-06-20 00:00:10'
            span=211000 ; 2006-06-22 10:22:49
            format = "TSD3_LW"
        END
        'setee07_ss501_sw_gb002_sglitch': BEGIN
            time='2006-06-15 00:00:10'
            span=211000 ; 2006-06-17 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb002_sglitch': BEGIN
            time='2006-06-15 00:00:10'
            span=211000 ; 2006-06-17 10:22:49
            format = "TSD3_LW"
        END
        'setee07_ss501_sw_gb002': BEGIN
            time='2006-06-10 00:00:10'
            span=211000 ; 2006-06-12 10:22:49
            format = "TSD3_SW"
        END
        'setee07_ss502_lw_gb002': BEGIN
            time='2006-06-10 00:00:10'
            span=211000 ; 2006-06-12 10:22:49
            format = "TSD3_LW"
        END


        ;;;; ee06
        'setee06_ss501_sw_gb001_p2p_sglitch': BEGIN
            time='2006-06-04 00:00:10'
            span=211000 ; 2006-06-06 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb001_p2p_sglitch': BEGIN
            time='2006-06-04 00:00:10'
            span=211000 ; 2006-06-06 10:22:49
            format = "TSD3_LW"
        END
        'setee06_ss501_sw_gb001_p2p': BEGIN
            time='2006-05-30 00:00:10'
            span=211000 ; 2006-06-01 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb001_p2p': BEGIN
            time='2006-05-30 00:00:10'
            span=211000 ; 2006-06-01 10:22:49
            format = "TSD3_LW"
        END
        'setee06_ss501_sw_gb001_sglitch': BEGIN
            time='2006-05-25 00:00:10'
            span=211000 ; 2006-05-27 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb001_sglitch': BEGIN
            time='2006-05-25 00:00:10'
            span=211000 ; 2006-05-27 10:22:49
            format = "TSD3_LW"
        END
        'setee06_ss501_sw_gb001': BEGIN
            time='2006-05-20 00:00:10'
            span=211000 ; 2006-05-22 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb001': BEGIN
            time='2006-05-20 00:00:10'
            span=211000 ; 2006-05-22 10:22:49
            format = "TSD3_LW"
        END

        'setee06_ss501_sw_gb002_p2p_sglitch': BEGIN
            time='2006-05-15 00:00:10'
            span=211000 ; 2006-05-17 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb002_p2p_sglitch': BEGIN
            time='2006-05-15 00:00:10'
            span=211000 ; 2006-05-17 10:22:49
            format = "TSD3_LW"
        END
        'setee06_ss501_sw_gb002_p2p': BEGIN
            time='2006-05-10 00:00:10'
            span=211000 ; 2006-05-12 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb002_p2p': BEGIN
            time='2006-05-10 00:00:10'
            span=211000 ; 2006-05-12 10:22:49
            format = "TSD3_LW"
        END
        'setee06_ss501_sw_gb002_sglitch': BEGIN
            time='2006-05-06 00:00:10'
            span=211000 ; 2006-05-08 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb002_sglitch': BEGIN
            time='2006-05-06 00:00:10'
            span=211000 ; 2006-05-08 10:22:49
            format = "TSD3_LW"
        END
        'setee06_ss501_sw_gb002': BEGIN
            time='2006-05-01 00:00:10'
            span=211000 ; 2006-05-03 10:22:49
            format = "TSD3_SW"
        END
        'setee06_ss502_lw_gb002': BEGIN
            time='2006-05-01 00:00:10'
            span=211000 ; 2006-05-03 10:22:49
            format = "TSD3_LW"
        END


        ;; ee05
        'setee05_ss506_sw_gb002_sglitch': BEGIN
            time='2006-10-20 00:00:00'
            span=302400 ; 2006-10-23...
            format = "TSD3_SW"
        END
        'setee05_ss506_lw_gb002_sglitch': BEGIN
            time='2006-10-20 00:00:00'
            span=302400 ; 2006-10-23...
            format = "TSD3_LW"
        END
        'setee05_ss506_sw_gb002': BEGIN
            time='2006-10-15 00:00:00'
            span=302400 ; 2006-10-18...
            format = "TSD3_SW"
        END
        'setee05_ss506_lw_gb002': BEGIN
            time='2006-10-15 00:00:00'
            span=302400 ; 2006-10-18...
            format = "TSD3_LW"
        END
        'setee05_ss505_sw_gb002_sglitch': BEGIN
            time='2006-10-06 00:00:00'
            span=302400 ; 2006-10-08...
            format = "TSD3_SW"
        END
        'setee05_ss505_lw_gb002_sglitch': BEGIN
            time='2006-10-06 00:00:00'
            span=302400 ; 2006-10-08...
            format = "TSD3_LW"
        END
        'setee05_ss505_sw_gb002': BEGIN
            time='2006-10-01 00:00:00'
            span=302400 ; 2006-10-03...
            format = "TSD3_SW"
        END
        'setee05_ss505_lw_gb002': BEGIN
            time='2006-10-01 00:00:00'
            span=302400 ; 2006-10-03 
            format = "TSD3_LW"
        END


        ;; ee04
        'setee04_ss500_sw_gb002': BEGIN
            time='2006-04-01 00:00:10'
            span=211000 ; 2006-04-03 10:22:49
            format = "TSD3_SW"
        END
        'setee04_ss500_lw_gb002': BEGIN
            time='2006-04-01 00:00:10'
            span=211000 ; 2006-04-03 10:22:49
            format = "TSD3_LW"
        END


        ;; ee03
        'setee03_ss501_sw_gb001_sglitch': BEGIN
            time='2006-03-25 00:00:10'
            span=211000 ; 2006-03-27 10:22:49
            format = "TSD3_SW"
        END
        'setee03_ss502_lw_gb001_sglitch': BEGIN
            time='2006-03-25 00:00:10'
            span=211000 ; 2006-03-27 10:22:49
            format = "TSD3_LW"
        END
        'setee03_ss501_sw_gb001': BEGIN
            time='2006-03-20 00:00:10'
            span=211000 ; 2006-03-22 10:22:49
            format = "TSD3_SW"
        END
        'setee03_ss502_lw_gb001': BEGIN
            time='2006-03-20 00:00:10'
            span=211000 ; 2006-03-22 10:22:49
            format = "TSD3_LW"
        END
        'setee03_ss501_sw_gb002_sglitch': BEGIN
            time='2006-03-15 00:00:10'
            span=211000 ; 2006-03-17 10:22:49
            format = "TSD3_SW"
        END
        'setee03_ss502_lw_gb002_sglitch': BEGIN
            time='2006-03-15 00:00:10'
            span=211000 ; 2006-03-17 10:22:49
            format = "TSD3_LW"
        END
        'setee03_ss501_sw_gb002': BEGIN
            time='2006-03-10 00:00:10'
            span=211000 ; 2006-03-12 10:22:49
            format = "TSD3_SW"
        END
        'setee03_ss502_lw_gb002': BEGIN
            time='2006-03-10 00:00:10'
            span=211000 ; 2006-03-12 10:22:49
            format = "TSD3_LW"
        END


        ;; ee02
        'setee02_ss501_sw_gb002_sglitch': BEGIN
            time='2006-03-02 00:00:10'
            span=211000 ; 2006-03-04 10:22:49
            format = "TSD3_SW"
        END
        'setee02_ss502_lw_gb002_sglitch': BEGIN
            time='2006-03-02 00:00:10'
            span=211000 ; 2006-03-04 10:22:49
            format = "TSD3_LW"
        END
        'setee02_ss501_sw_gb002': BEGIN
            time='2006-02-25 00:00:10'
            span=211000 ; 2006-02-27 10:22:49
            format = "TSD3_SW"
        END
        'setee02_ss502_lw_gb002': BEGIN
            time='2006-02-25 00:00:10'
            span=211000 ; 2006-02-27 10:22:49
            format = "TSD3_LW"
        END


        ;; ee01
        'setee01_ss501_sw_gb002': BEGIN
            time='2006-02-10 00:00:10'
            span=211000 ; 2006-02-12 10:22:49
            format = "TSD3_SW"
        END
        'setee01_ss502_lw_gb002': BEGIN
            time='2006-02-10 00:00:10'
            span=211000 ; 2006-02-12 10:22:49
            format = "TSD3_LW"
        END
        'setee01_ss500_sw_gb002': BEGIN
            time='2006-02-05 00:00:10'
            span=211000 ; 2006-02-07 10:22:49
            format = "TSD3_SW"
        END
        'setee01_ss500_lw_gb002': BEGIN
            time='2006-02-05 00:00:10'
            span=211000 ; 2006-02-07 10:22:49
            format = "TSD3_LW"
        END
        'setee01_ss499_sw_gb002': BEGIN
            time='2006-02-01 00:00:10'
            span=211000 ; 2006-02-03 10:22:49
            format = "TSD3_SW"
        END
        'setee01_ss499_lw_gb002': BEGIN
            time='2006-02-01 00:00:10'
            span=211000 ; 2006-02-03 10:22:49
            format = "TSD3_LW"
        END


        ;;;; Required data sets from IT#2 for IT#3 test week

;         ;; set18
;         'set18_ss00_sw_sglitch3': BEGIN
;             time='2005-08-01 10:00:10'
;             span= 600 ; 2005-08-01 10:07:41
;             format = "TSD3_SW"
;         END
;         'set18_ss00_lw_sglitch3': BEGIN
;             time='2005-08-01 10:00:10'
;             span= 600 ; 2005-08-01 10:07:41
;             format = "TSD3_LW"
;         END
;         'set18_ss00_sw_sglitch2': BEGIN
;             time='2005-08-01 08:20:10'
;             span= 600 ; 2005-08-01 08:27:41
;             format = "TSD3_SW"
;         END
;         'set18_ss00_lw_sglitch2': BEGIN
;             time='2005-08-01 08:20:10'
;             span= 600 ; 2005-08-01 08:27:41
;             format = "TSD3_LW"
;         END
;         'set18_ss00_sw_sglitch1': BEGIN
;             time='2005-08-01 06:40:10'
;             span= 600 ; 2005-08-01 06:47:41
;             format = "TSD3_SW"
;         END
;         'set18_ss00_lw_sglitch1': BEGIN
;             time='2005-08-01 06:40:10'
;             span= 600 ; 2005-08-01 08:47:41
;             format = "TSD3_LW"
;         END
;         'set18_ss00_sw_glitch3': BEGIN
;             time='2005-08-01 05:00:10'
;             span= 600 ; 2005-08-01 05:07:41
;             format = "TSD3_SW"
;         END
;         'set18_ss00_lw_glitch3': BEGIN
;             time='2005-08-01 05:00:10'
;             span= 600 ; 2005-08-01 05:07:41
;             format = "TSD3_LW"
;         END
;         'set18_ss00_sw_glitch2': BEGIN
;             time='2005-08-01 03:20:10'
;             span= 600 ; 2005-08-01 03:27:41
;             format = "TSD3_SW"
;         END
;         'set18_ss00_lw_glitch2': BEGIN
;             time='2005-08-01 03:20:10'
;             span= 600 ; 2005-08-01 03:27:41
;             format = "TSD3_LW"
;         END
;         'set18_ss00_sw_glitch1': BEGIN
;             time='2005-08-01 01:40:10'
;             span= 600 ; 2005-08-01 01:47:41
;             format = "TSD3_SW"
;         END
;         'set18_ss00_lw_glitch1': BEGIN
;             time='2005-08-01 01:40:10'
;             span= 600 ; 2005-08-01 01:47:41
;             format = "TSD3_LW"
;         END


        ;; set17
        'set17_sw': BEGIN
            time='2005-07-10 00:00:00'
            span= 4300 ; 2005-07-10 01:09:10
            format = "TSD3_SW"
        END
        'set17_lw': BEGIN
            time='2005-07-10 00:00:00'
            span= 4300 ; 2005-11-01 01:09:10
            format = "TSD3_LW"
        END
;        'set17_sw_bb': BEGIN
;            time='2005-07-01 01:40:00'
;            span= 4300 ; 2005-07-01 02:49:10
;            format = "TSD3_SW"
;        END
;        'set17_lw_bb': BEGIN
;            time='2005-07-01 01:40:00'
;            span= 4300 ; 2005-11-01 02:49:10
;            format = "TSD3_LW"
;        END


        ;; set12
        'set12_sw': BEGIN
            time='2005-02-10 00:00:00'
            span= 6000-5 ; 2005-02-10 01:39:53
            format = "TSD3_SW"
        END
        'set12_lw': BEGIN
            time='2005-02-10 00:00:00'
            span= 6000-5 ; 2005-02-10 01:39:53
            format = "TSD3_LW"
        END


        ;; set01
        'set1b_lw': BEGIN
            time='2004-01-01 03:20:00'
            span= 6000-5 ; 2004-01-01 04:59:55
            format = "TSD3_LW"
        END
        'set1b_sw': BEGIN
            time='2004-01-01 03:20:10'
            span= 6000-5 ; 2004-01-01 04:59:53
            format = "TSD3_SW"
        END
        'set1a_lw': BEGIN
            time='2004-01-01 00:00:00'
            span= 6000-5 ; 2004-01-01 01:39:55
            format = "TSD3_LW"
        END
        'set1a_sw': BEGIN
            time='2004-01-01 00:00:10'
            span= 6000-5 ; 2004-01-01 01:39:53
            format = "TSD3_SW"
        END


        ELSE: LMESSAGE, 0, 'Unsupported datasetid: ' + datasetid
    ENDCASE

    IF NOT KEYWORD_SET(gb) THEN gb = 0
    IF NOT KEYWORD_SET(infoonly) THEN infoonly = 0
    IF NOT KEYWORD_SET(compress) THEN compress = 0
    IF NOT KEYWORD_SET(keep) THEN keep = 0

    ; debug message
    MESSAGE, 'Query condition of Get_TestDataSet_From_LSD("' + datasetid + '") has been interpreted as follows:', /INFORMATIONAL, /NONAME, /NOPREFIX
;    MESSAGE, '  TIME="' + time + '", SPAN="' + span + '", FORMAT="' + format +'"', /INFORMATIONAL, /NONAME, /NOPREFIX

    ; Get TSDARR and return
;    RETURN, Get_TSDARR_From_LDS(MODE='sim', TIME=time, SPAN=span, FORMAT=format, GB_PROCESSED=gb, COMPRESS=compress, KEEP=keep)

    IF infoonly EQ 1 THEN BEGIN
        RETURN, Get_INFOARR_From_LDS(MODE='sim', TIME=time, SPAN=span, FORMAT=format, INDEX=index, GB_PROCESSED=gb, COMPRESS=compress, KEEP=keep)
    ENDIF ELSE BEGIN
        RETURN, Get_TSDARR_From_LDS(MODE='sim', TIME=time, SPAN=span, FORMAT=format, INDEX=index, GB_PROCESSED=gb, COMPRESS=compress, KEEP=keep)
    ENDELSE
END
