; $Id: tsd__define.pro,v 1.53 2007/01/22 13:21:01 cyamauch Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; CLASS_NAME:
;   TSD
;
; PURPOSE:
;   Define the object structure for a Time Series Data (TSD) of FIS Survey.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   tsdobj = OBJ_NEW('tsd')
;
; INPUTS:
;   'tsd'   Name of object class
;
; OUTPUTS:
;   tsdobj  Name of object instance
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   See chapter 'Object Basics' in 'Building IDL Applications' in detail.
;
; DESCRIPTION:
;   See 'ASTRO-F FIS/Survey Data Structure' for data structure in detail.
;
; EXAMPLE:
;   See calling sequence above.
;
; SIDE EFFECTS:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, June 2002.
;
;-

;----------------------------------------------------------------------------
; public methods
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Read_TSD('infile.fits' [, RANGE=range])
;
; INPUTS:
;   infile  Input filename of TSD (which is a FITS format) (STRING)
;
; OPTIONAL KEYWORD:
;   range       A scalar or two element vector giving the start
;               and end rows to be retrieved.  The first row is row 0.
;               If only a single value, x, is given in the range,
;               the range is assumed to be [0,x-1].
;               See comment of MRDFITS of AstroLib for more detail.
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;
; EXAMPLE:
;   By default, all of infile is red into TSD internal structure on
;   IDL memory.
;
;     IDL> tsd = OBJ_NEW('tsd')
;     IDL> dummy = tsd->Read_TSD('infile.fits')
;
;   If the input file is so large and you don't have enough memory, you
;   also can get a subset of the file by specifing range you want to read.
;   Please note that range overflow might be occured.
;
;     IDL> tsd = OBJ_NEW('tsd')
;     IDL> range = [100,200]  ; Note that nrow=101
;     IDL> dummy = tsd->Read_TSD('so_laaaaaarge.fits', RANGE=range)
;     IDL> aftime = tsd->Get_Value_TSD('FIS_OBS', 'AFTIME')
;     IDL> help, aftime
;     AFTIME          INT       = Array[101]
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Read_TSD, infile, RANGE=range, VERSION=version
    ; Define an error handler first.
    ; See http://www.dfanning.com/tips/catch_trace.html for more detail.
    CATCH, error_status
    IF error_status NE 0 THEN BEGIN
        CATCH, /CANCEL
        HELP, /LAST_MESSAGE, OUTPUT=errmsg
        FOR i=0,N_ELEMENTS(errmsg)-1 DO PRINT, errmsg[i]
        ; STOP ; or EXIT
        RETURN, !FAIL
    ENDIF

    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT FILE_TEST(infile, /READ, /REGULAR) THEN BEGIN
        LMESSAGE, 0, infile + ': No such file or directory.', /CONTINUE
        RETURN, !FAIL
    ENDIF
    IF FILE_TEST(infile, /READ, /DIRECTORY) THEN BEGIN
        LMESSAGE, 0, infile + ': is a directory, not a file.', /CONTINUE
        RETURN, !FAIL
    ENDIF

    LMESSAGE, 5, 'Read TSD file: ' + infile, /PRINT

    ; Strip directory name of infile
    filename = STRMID(infile, STRPOS(infile, '/', /REVERSE_SEARCH)+1)
    dummy = self->Set_Filename(filename)

    ;  file open check
    OpenR, lun, infile, /BLOCK, /GET_LUN, ERROR=error, /COMPRESS
    IF error NE 0 THEN BEGIN
        message='Error opening ' + infile + '. '
        IF N_ELEMENTS(errmsg) NE 0 THEN BEGIN
            errmsg = message
            RETURN, !FAIL
        END ELSE LMESSAGE, 0, message + !ERROR_STATE.MSG
    ENDIF

    ; read primary HDU first
    IF self.primary->read(lun) EQ !FAIL THEN $
        LMESSAGE, 0, 'Primary HDU read error:' + infile

    ; get datatype and make extension objects
    instrume = self.primary->Get_Header_Keyword('INSTRUME')
    self.instrume = STRUPCASE(STRTRIM(instrume,2))
    fmttype = self.primary->Get_Header_Keyword('FMTTYPE')
    IF STRMATCH(STRTRIM(STRING(fmttype),2), '0') THEN BEGIN
        datatype = self.primary->Get_Header_Keyword('DATATYPE')
    ENDIF ELSE BEGIN
        s = STRTRIM(fmttype,2)
        datatype = '';
        IF STRMATCH(s,'ASTRO-F TSD FIS_SW') THEN datatype = 'FIS_SW'
        IF STRMATCH(s,'ASTRO-F TSD FIS_LW') THEN datatype = 'FIS_LW'
        IF STRMATCH(s,'ASTRO-F TSD ISO') THEN datatype = 'ISO'
    ENDELSE
    self.datatype = STRUPCASE(STRTRIM(datatype,2))
    tsdver = FIX(self.primary->Get_Header_Keyword('FTYPEVER'))
    IF tsdver EQ 0 THEN tsdver = FIX(self.primary->Get_Header_Keyword('TSDVER'))
    IF tsdver EQ 0 THEN tsdver = 1
    IF KEYWORD_SET(version) THEN tsdver = version
    LMESSAGE, 5, '  INSTRUME=' + instrume + ', DATATYPE=' + datatype + ', TSDVER=' + STRTRIM(STRING(tsdver),2), /PRINT

    dummy = self->createExtObj(VERSION=tsdver)

    ; Read extension HDU
    ; If it fails, an exception is generated by MESSAGE procedure
    ; and error_status is set to 1
    extnames = self->Get_Extnames(VERSION=tsdver)
    FOR i=1,N_ELEMENTS(extnames) DO BEGIN
        dummy = self.(i)->read(lun, tsdver, RANGE=range)
    ENDFOR

    ; Close the binary table extension
;    CLOSE, lun
    FREE_LUN, lun

    RETURN, !SUCCEED
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Write_TSD('outfile.fits')
;
; INPUTS:
;   outfile Output filename of TSD (which is a FITS format) (STRING)
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Write_TSD, outfile, VERSION=version
    ; Define an error handler first.
    ; See http://www.dfanning.com/tips/catch_trace.html for more detail.
    CATCH, error_status
    IF error_status NE 0 THEN BEGIN
        CATCH, /CANCEL
        HELP, /LAST_MESSAGE, OUTPUT=errmsg
        FOR i=0,N_ELEMENTS(errmsg)-1 DO PRINT, errmsg[i]
        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF

    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    LMESSAGE, 5, 'Write TSD file: ' + outfile, /PRINT

    LMESSAGE, 5, '  INSTRUME = ' + self.instrume, /PRINT
    LMESSAGE, 5, '  DATATYPE = ' + self.datatype, /PRINT
    tsdver = FIX(self.primary->Get_Header_Keyword('FTYPEVER'))
    IF tsdver EQ 0 THEN tsdver = FIX(self.primary->Get_Header_Keyword('TSDVER'))
    IF tsdver EQ 0 THEN tsdver = 1
    IF KEYWORD_SET(version) THEN tsdver = version
    LMESSAGE, 5, '  TSDVER   = ' + STRING(tsdver), /PRINT

    ; Get 'YYYY-MM-DDThh:mm:ss' style date using Astrolib GET_DATE
    GET_DATE, crtdate, /TIMETAG
    tst = self.primary->Get_Header_Keyword('DATE')
    IF STRMATCH(STRTRIM(STRING(tst),2), '0') THEN dummy = self.primary->Set_Header_Keyword('DATE', crtdate)
    ; Add history
    msg = SYSTIME(/UTC) + ':  Saved by TSD::Write_TSD()'
    dummy = self.primary->Set_Header_Keyword('HISTORY', msg)

    ; Write primary data
    IF self.primary->write(outfile) NE !SUCCEED THEN $
        LMESSAGE, 0, 'Primary HDU write error:' + outfile

    ; Write extension data
    extnames = self->Get_Extnames(VERSION=version)
    FOR i=1,N_ELEMENTS(extnames) DO BEGIN
        dummy = self.(i)->write(outfile)
    ENDFOR

    RETURN, !SUCCEED
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Create_TSD(datatype, naxis2)
;
; INPUTS:
;   datatype    Datatype (STRING)
;               (now available: 'FIS_LW' and 'FIS_SW')
;   naxis2  Number of rows of binary table (INT)
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Create_TSD, datatype, naxis2, VERSION=version, INSTRUME=instrume
    ; Define an error handler first.
    ; See http://www.dfanning.com/tips/catch_trace.html for more detail.
    CATCH, error_status
    IF error_status NE 0 THEN BEGIN
        CATCH, /CANCEL
        HELP, /LAST_MESSAGE, OUTPUT=errmsg
        FOR i=0,N_ELEMENTS(errmsg)-1 DO PRINT, errmsg[i]
        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    DPRINT, 'Create TSD file: '

    IF LONG(naxis2) LE 0 THEN MESSAGE, 'Error: NAXIS2 must be greater than 1'
    IF KEYWORD_SET(instrume) THEN BEGIN
        self.instrume = STRUPCASE(STRTRIM(instrume,2))
    ENDIF
    DPRINT, 'INSTRUME = ' + self.instrume
    IF KEYWORD_SET(datatype) THEN BEGIN
        self.datatype = STRUPCASE(STRTRIM(datatype,2))
    ENDIF
    DPRINT, 'DATATYPE = ' + self.datatype
    tsdver = 4
    IF KEYWORD_SET(version) THEN tsdver = version
    DPRINT, 'TSDVER   = ' + STRING(tsdver)

    IF self->createExtObj(VERSION=version) EQ !FAIL THEN MESSAGE, 'Error: '+self.datatype

    IF self.primary->create(self.datatype, naxis2,INSTRUME=instrume) NE !SUCCEED THEN MESSAGE, 'Create error'

    extnames = self->Get_Extnames(VERSION=version)
    FOR i=1,N_ELEMENTS(extnames) DO BEGIN
        dummy = self.(i)->create(naxis2)
    ENDFOR

    ; Add history
    msg = SYSTIME(/UTC) + ':  Created by TSD::Create_TSD()'
    dummy = self.primary->Set_Header_Keyword('HISTORY', msg)

    RETURN, !SUCCEED
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   filename = tsd->Read_TSD_From_LDS(MODE=mode, TIME=time, SPAN=span, $
;                FORMAT=format, KEEP=keep, COMPRESS=compress)
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   MODE    Data query mode: ("Lab" or "Sim": default: "Lab")
;   TIME    Start time of request data
;     Time should be JST for Lab data, and UTC for Sim data.
;               (STRING FORMAT: '2004-07-10 23:40:00')
;   SPAN    Span of request data (seconds)
;   FORMAT  Output Format
;		(now available: 'DAT','TSD4_LW','TSD4_SW',
;		 'TSD3_LW','TSD3_SW','TSD2_LW','TSD2_SW')
;   KEEP    Keep temporarily downloaded file if set.
;     (note that on-memory object is not removed.)
;   COMPRESS    Get a compressed (gzipped) data or not (default: no)
;
; KEYWORDS TO BE IMPLEMENTED:
;   RA   Right Ascension
;   DEC     Declination
;   EQUINOX Equinox (Default: J2000)
;
; OUTPUTS:
;   filename    Name of filename (STRING)
;     Return !FAIL if fail.
;
; EXAMPLE:
;   You can access to LDS interactively via: !LDS_URL
;   This method is for non-interactive use.
;
;     IDL> tsd = OBJ_NEW('tsd')
;     IDL> dummy = tsd->Read_TSD_From_LDS(TIME='2004-07-10 23:40:00', SPAN=10, FORMAT='TSD4_LW')
;     IDL> aftime = tsd->Get_Value_TSD('FIS_OBS', 'AFTIME')
;     IDL> help, aftime
;     AFTIME          INT       = Array[473]
;
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Read_TSD_From_LDS_orig, MODE=mode, TIME=time, SPAN=span, FORMAT=format, KEEP=keep, COMPRESS=compress
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    IF NOT KEYWORD_SET(time) OR NOT KEYWORD_SET(span) OR NOT KEYWORD_SET(format) THEN BEGIN
        MESSAGE, "Keyword 'TIME' and 'SPAN' should be set.", /CONTINUE
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(mode) THEN mode = "Lab"
    available_modes = ["LAB", "SIM"]
    tmp = WHERE(STRMATCH(available_modes, STRUPCASE(mode), /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        MESSAGE, 'No such mode: ' + mode, /CONTINUE
        MESSAGE, 'Available modes are: ' + $
                 STRJOIN(available_modes,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(format) THEN format = "DAT"
    available_formats = ['DAT', 'TSD4_LW', 'TSD4_SW', 'TSD3_LW', 'TSD3_SW', 'TSD2_LW', 'TSD2_SW']
    tmp = WHERE(STRMATCH(available_formats, STRUPCASE(format), /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        MESSAGE, 'No such format: ' + format, /CONTINUE
        MESSAGE, 'Available formats are: ' + $
                 STRJOIN(available_formats,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    Query_URL = !LDS_URL + "lds.cgi?.State=Download_Direct" + $
                "&.Mode=" + self->URLencode(STRING(mode)) + $
                "&time=" + self->URLencode(STRING(time)) + $
                "&span=" + self->URLencode(STRING(span)) + $
                "&format=" + self->URLencode(STRING(format))
    IF KEYWORD_SET(compress) THEN Query_URL = Query_URL + "&gzip=1"

    ; make temporary file name with PSUEDO_TMPNAM() in util.pro
    tmpfile = GETENV('IDL_TMPDIR') + self->PSUEDO_TMPNAM()
    IF FILE_TEST(tmpfile) THEN FILE_DELETE, tmpfile
;    DPRINT, tmpfile

    ; Post query, get FITS file from LDS server, and store it into tmpfile
    self.filename = LDS_WEBGET(Query_URL, COPYFILE=tmpfile)
;    DPRINT, self.filename
    IF self.filename EQ '' THEN BEGIN
        MESSAGE, '404 Not Found: The requested data was not found on the LDS.', /CONTINUE
        RETURN, !FAIL
    ENDIF

    ; check whether fits file is header only or not
    OpenR, lun, tmpfile, /BLOCK, /GET_LUN, ERROR=error, /COMPRESS
    IF error NE 0 THEN BEGIN
        message='Error opening ' + infile + '. '
        IF N_ELEMENTS(errmsg) NE 0 THEN BEGIN
            errmsg = message
            RETURN, !FAIL
        END ELSE MESSAGE, message + !ERROR_STATE.MSG
    ENDIF
    result = FSTAT(lun)
;    DPRINT, result.SIZE
    IF result.SIZE LE 22 THEN BEGIN ; touch tmp; gzip tmp; ls -l tmp.gz -> 22
        MESSAGE, 'No match data in LDS.', /CONTINUE
        MESSAGE, 'Conditions: ' + STRJOIN(STRING([time,span,format]),', '), /CONTINUE
        MESSAGE, 'Please Check ' + LDS_Path, /CONTINUE
        FREE_LUN, lun
        FILE_DELETE, tmpfile
;        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    CLOSE, lun
    FREE_LUN, lun

;    FILE_MOVE, tmpfile, GETENV('IDL_TMPDIR') + self.filename, /OVERWRITE
    self.filename = tmpfile
    IF STRMATCH(STRMID(format,0,3), "TSD", /FOLD_CASE) THEN BEGIN
        version = FIX(STRMID(format, 3, 1))
        dummy = self->Read_TSD(self.filename, VERSION=version)
    ENDIF
    IF KEYWORD_SET(KEEP) NE 1 THEN FILE_DELETE, tmpfile

    ; null pointer, something wrong (maybe no match data)
    IF PTR_VALID(self.fis_obs->Get_Data(/POINTER)) NE 1 THEN BEGIN
        MESSAGE, 'NULL pointer: mayby no match data in LDS.', /CONTINUE
        MESSAGE, 'Conditions: ' + STRJOIN(STRING([time,span,format]),', '), /CONTINUE
        MESSAGE, 'Please Check ' + LDS_Path, /CONTINUE
        FREE_LUN, lun
        FILE_DELETE, tmpfile
        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    RETURN, self.filename
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   filename = tsd->Read_TSD_From_LDS(MODE=mode, TIME=time, SPAN=span, $
;                FORMAT=format, KEEP=keep, COMPRESS=compress)
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   MODE    Data query mode: ("Lab" or "Sim" or "Fli": default: "Lab")
;   TIME    Start time of request data
;     Time should be JST for Lab data, and UTC for Sim data.
;               (STRING FORMAT: '2004-07-10 23:40:00')
;   SPAN    Span of request data (seconds)
;   FORMAT  Output Format
;		(now available: 'DAT','TSD4_LW','TSD4_SW',
;		 'TSD3_LW','TSD3_SW','TSD2_LW','TSD2_SW')
;   GB_PROCESSED Flag whether data is processed by GreenBox or not
;               (0: not_processed, 1: processed.  Default: 0)
;   KEEP    Keep temporarily downloaded file if set.
;     (note that on-memory object is not removed.)
;   COMPRESS    Get a compressed (gzipped) data or not (default: no)
;
; KEYWORDS TO BE IMPLEMENTED:
;   RA   Right Ascension
;   DEC     Declination
;   EQUINOX Equinox (Default: J2000)
;
; OUTPUTS:
;   filename    Name of filename (STRING)
;     Return !FAIL if fail.
;
; EXAMPLE:
;   You can access to LDS interactively via: !LDS_URL
;   This method is for non-interactive use.
;
;     IDL> tsd = OBJ_NEW('tsd')
;     IDL> dummy = tsd->Read_TSD_From_LDS(TIME='2004-07-10 23:40:00', SPAN=10, FORMAT='TSD4_LW')
;     IDL> aftime = tsd->Get_Value_TSD('FIS_OBS', 'AFTIME')
;     IDL> help, aftime
;     AFTIME          INT       = Array[473]
;
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Read_TSD_From_LDS, MODE=mode, TIME=time, SPAN=span, DATATYPE=datatype, GB_PROCESSED=gb, COMPRESS=compress, KEEP=keep, FORMAT=format
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    IF NOT KEYWORD_SET(time) OR NOT KEYWORD_SET(span) THEN BEGIN
        MESSAGE, "Keyword 'TIME' and 'SPAN' should be set.", /CONTINUE
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(mode) THEN mode = "Fli"
    available_modes = ["LAB", "SIM", "FLI"]
    tmp = WHERE(STRMATCH(available_modes, STRUPCASE(mode), /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        MESSAGE, 'No such mode: ' + mode, /CONTINUE
        MESSAGE, 'Available modes are: ' + $
                 STRJOIN(available_modes,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(datatype) THEN BEGIN
        IF KEYWORD_SET(format) THEN datatype=format
    ENDIF

    IF NOT KEYWORD_SET(datatype) THEN BEGIN
        MESSAGE, 'Specify datatype.', /CONTINUE
        RETURN, !FAIL
    ENDIF

    IF NOT KEYWORD_SET(gb) THEN gb = 0
    IF NOT KEYWORD_SET(compress) THEN compress = 0
    IF NOT KEYWORD_SET(keep) THEN keep = 0

    DPRINT, 'LDS_URL: ' + !LDS_URL
    DPRINT, 'Read TSD from LDS with following condition:'
    DPRINT, 'mode="' + mode + '", time="' + time + '", span="' + STRTRIM(STRING(span),2) + '", datatype="' + datatype + '"'

    ; Download TSD files from LDS via XMLRPC interface
    xmlrpc = OBJ_NEW('xmlrpc')
;    proxy = !LDS_URL + 'xmlrpc.cgi'
;    method = 'LDS.downloadViaXMLRPC'
    ;;proxy = !LDS_URL + 'xmlrpc_payload.cgi'
    ;;method = 'LDS.downloadViaXMLRPC_Mini_Payload'
    proxy = !LDS_URL + 'nph-xmlrpc_oreload.cgi'
    method = 'LDS.downloadViaXMLRPC_Oreload'
    ra2000 = -999
    de2000 = -999
    size = 0
    params = { PARAMS, mode: mode, time: time, span: span, $
               ra2000: ra2000, de2000: de2000, size: size, $
               format: datatype, gb_processed: gb, gzip: compress }
    tmpfile = GETENV('IDL_TMPDIR') + self->PSUEDO_TMPNAM()
    IF FILE_TEST(tmpfile) THEN FILE_DELETE, tmpfile
    xmlrpc = xmlrpc->proxy(proxy)
    xmlrpc = xmlrpc->call(method, params, tmpfile)
    IF NOT PTR_VALID(xmlrpc.params) THEN BEGIN
        MESSAGE, 'XMLRPC ERROR: ', /CONTINUE
        OBJ_DESTROY, xmlrpc
        ; STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    result = xmlrpc->result()

    IF xmlrpc->isFault() EQ 1 THEN BEGIN
        MESSAGE, 'XMLRPC ERROR: ', /CONTINUE
        MESSAGE, 'faultCode:   ' + STRING(result.faultCode), /CONTINUE
        MESSAGE, 'faultString: ' + result.faultString, /CONTINUE
        OBJ_DESTROY, xmlrpc
        FILE_DELETE, tmpfile
        ; STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    OBJ_DESTROY, xmlrpc

    named_tmpfile = GETENV('IDL_TMPDIR') + result.param_0.(0).filename
    IF FILE_TEST(named_tmpfile) THEN FILE_DELETE, named_tmpfile

    ; rename the temp. file
    FILE_MOVE, tmpfile, named_tmpfile

    DPRINT, 'Saved temporary TSD file: ' + named_tmpfile

    self.filename = named_tmpfile
    dummy = self->Read_TSD(self.filename)
    IF KEYWORD_SET(KEEP) NE 1 THEN FILE_DELETE, named_tmpfile

    ; null pointer, something wrong (maybe no match data)
    IF PTR_VALID(self.fis_obs->Get_Data(/POINTER)) NE 1 THEN BEGIN
        MESSAGE, 'NULL pointer: mayby no match data in LDS.', /CONTINUE
        MESSAGE, 'Conditions: ' + STRJOIN(STRING([time,span,datatype]),', '), /CONTINUE
;        MESSAGE, 'Please Check ' + LDS_Path, /CONTINUE
        FREE_LUN, lun
        FILE_DELETE, named_tmpfile
        STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    RETURN, self.filename
END


;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Write_TSD_To_LDS(/KEEP)
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   /KEEP   Keep temporarily created file if set. (just for debug)
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;
; EXAMPLE:
;   You can access to LDS interactively via: !LDS_URL
;   This method is for non-interactive use.
;
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Write_TSD_To_LDS, KEEP=keep
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    ; write tsddata to temporary file
    tmpfile = GETENV('IDL_TMPDIR') + self->PSUEDO_TMPNAM()
    IF FILE_TEST(tmpfile) THEN FILE_DELETE, tmpfile
    status = self->Write_TSD(tmpfile)
    IF status EQ !FAIL THEN RETURN, !FAIL
    ; read tmpfile to tmpdata
    OPENR, lun, tmpfile, /Get_LUN
    res = FSTAT(lun)
    tmpdata = BYTARR(res.SIZE, /NOZERO)
    READU, lun, tmpdata
    FREE_LUN, lun
    IF KEYWORD_SET(KEEP) NE 1 THEN FILE_DELETE, tmpfile
    params = CREATE_STRUCT('instrume', self.instrume, 'filename', self->Get_Filename(), 'filedata', tmpdata)

    ; Upload TSD files to LDS via XMLRPC interface
    xmlrpc = OBJ_NEW('xmlrpc')
    proxy = !LDS_URL + 'xmlrpc.cgi'
    method = 'LDS.uploadViaXMLRPC'
    xmlrpc = xmlrpc->proxy(proxy)
    xmlrpc = xmlrpc->call(method, params, '', KEEP=keep)
    result = xmlrpc->result()
    IF xmlrpc->isFault() EQ 1 THEN BEGIN
        MESSAGE, 'XMLRPC ERROR: ', /CONTINUE
        MESSAGE, 'faultCode:   ' + STRING(result.faultCode), /CONTINUE
        MESSAGE, 'faultString: ' + result.faultString, /CONTINUE
        OBJ_DESTROY, xmlrpc
        ; STOP ; or EXIT
        RETURN, !FAIL
    ENDIF
    OBJ_DESTROY, xmlrpc

    RETURN, !SUCCEED
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   filename = tsd->Get_Filename()
;
; INPUTS:
;   None
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   filename    Name of filename (STRING)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Filename
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    RETURN, self.filename
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Filename(filename)
;
; INPUTS:
;   filename    Name of filename (STRING)
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Filename, filename
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    self.filename = filename
    RETURN, !SUCCEED
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   version = tsd->Get_TSDVER()
;
; INPUTS:
;   None
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   version Version number (INT)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_TSDVer
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    version = self.primary->Get_Header_Keyword('FTYPEVER')
    IF version EQ 0 THEN version = self.primary->Get_Header_Keyword('TSDVER')
    IF version EQ 0 THEN version = 1
    RETURN, version
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   datatype = tsd->Get_Datatype()
;
; INPUTS:
;   None
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   datatype    Datatype (STRING)
;               (now available: 'FIS_LW' and 'FIS_SW')
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Datatype
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    RETURN, self.datatype
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   members = tsd->Get_Value_Members_TSD(extname)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   members Names of members of extname (STRARR)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Value_Members_TSD, extname
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, ['']
    END
    RETURN, (*p)->Get_Value_Members()
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   members = tsd->Get_Status_Members_TSD(extname)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   members Names of status members of extname (STRARR)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Status_Members_TSD, extname
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, ['']
    END
    RETURN, (*p)->Get_Status_Members()
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   members = tsd->Get_Flag_Members_TSD(extname)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   members Names of flag members of extname (STRARR)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Flag_Members_TSD, extname
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, ['']
    END
    RETURN, (*p)->Get_Flag_Members()
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   members = tsd->Get_Pix_Flag_Members_TSD(extname)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   members Names of pix_flag members of extname (STRARR)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Pix_Flag_Members_TSD, extname
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, ['']
    END
    RETURN, (*p)->Get_Pix_Flag_Members()
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   members = tsd->Get_Quality_Members_TSD(extname)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS' only)
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   members Names of quality members of extname (STRARR)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Quality_Members_TSD, extname
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, ['']
    END
    RETURN, (*p)->Get_Quality_Members()
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   header = tsd->Get_Header_TSD(extname [, /POINTER])
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;
; OPTIONAL KEYWORDS:
;   POINTER Keyword whether return value is a pointer or not.
;
; OUTPUTS:
;   header  FITS Header (STRARR)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Header_TSD, extname, POINTER=pointer
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    ; Simply
    ;   RETURN, self.primary.header
    ; does not work with following error:
    ;   % Expression must be a structure in this context: <No name>.
    ;
    ; Then,
    ;   RETURN, (self.primary).header, or
    ;   RETURN, self.primary->Get_Header()
    ; goes well, but IDL warns as follows:
    ; % Temporary variables are still checked out - cleaning up...
    ;
    ; Mmm... RSI should fix them.

    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, ['']
    END
    RETURN, (*p)->Get_Header(POINTER=pointer)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Header_TSD(extname, header)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   header  FITS Header (STRARR)
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Header_TSD, extname, header
;    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    ; Check whether header is a STRARR or not.
    IF SIZE(header, /TNAME) NE 'STRING' $
      OR SIZE(header, /N_DIMENSIONS) EQ 0 THEN BEGIN
        MESSAGE, 'Argument header must be a STRARR.' ;, /CONTINUE
        RETURN, !FAIL
    ENDIF

    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Set_Header(header)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   value = tsd->Get_Header_Keyword_TSD(extname, keyword)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   keyword Header Keyword (STRING)
;     The first eight bytes of a header card image.
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   value   Value of header keyword
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Header_Keyword_TSD, extname, keyword
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Get_Header_Keyword(keyword)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Header_Keyword_TSD(extname, keyword, value)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   keyword Header Keyword (STRING)
;     The first eight bytes of a header card image.
;   value   Value of Keyword
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Header_Keyword_TSD, extname, keyword, value
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Set_Header_Keyword(keyword, value)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   valarray = tsd->Get_Value_TSD(extname, fname [, /POINTER])
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   fname   Field name of FITS Binary table (STRING)
;
; OPTIONAL KEYWORDS:
;   POINTER Keyword whether return value is a pointer or not.
;
; OUTPUTS:
;   valarray    Array of value (or pointer) in question
;
; DESCRIPTION:
;   Copy any value in data array specified by keywords into a simple
;   time sequence array.
;
; COMMENT:
;   There may be some functions which are frequently used at
;   certain occasion but practically equivalent with Get_Value_TSD().
;   See also Get_Status_TSD(), Get_Flag_TSD(), and Get_Pix_Flag_TSD().
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Value_TSD, extname, fname, POINTER=pointer
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Get_Value(fname, POINTER=pointer)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Value_TSD(extname, fname, val)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   fname   Field name of FITS Binary table (STRING)
;   val     Value
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;
; DESCRIPTION:
;   Write time-sequence data array into the field in TSD specified by
;   keyword.  Keyword and corresponding data type, and data field in
;   TSD must be all consistent, otherwise the process is failure.
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Value_TSD, extname, fname, val
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Set_Value(fname, val)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   valarray = tsd->Get_Status_TSD(extname, key)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   key         Name of status of the extension (STRING)
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   valarray    Array of value (or pointer) in question
;
; DESCRIPTION:
;   Inquire status in TSD specified by keywords.
;
; COMMENT
;   This function is equivalent as far as a status is corresponding
;   to a bit in TSD. In addition, this function will treat a status
;   expressed by combination of two or more status bits.
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Status_TSD, extname, key
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Get_Status(key)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Status_TSD(extname, key, val)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   key         Name of status of the extension (STRING)
;   val     Value
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;
; DESCRIPTION:
;   Set status bits in TSD.
;
; COMMENT:
;   As our definition is that status are those contained in the telemetry
;   data or other initial data and are not allowed to change by the
;   reduction process, this function is only used to create a new TSD.
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Status_TSD, extname, key, value
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Set_Status(key, value)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   valarray = tsd->Get_Flag_TSD(extname, key)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   key         Name of flag of the extension (STRING)
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   valarray    Array of value (or pointer) in question
;
; DESCRIPTION:
;   Inquire flag.  Flag keyword name are defined separately. This
;   function is practically equivalent with Get_Value_TSD().
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Flag_TSD, extname, key
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Get_Flag(key)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Flag_TSD(extname, key, val)
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   key         Name of flag of the extension (STRING)
;   val     Value
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Flag_TSD, extname, key, value
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Set_Flag(key, value)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   valarray = tsd->Get_Pix_Flag_TSD(extname, key [, PIXNO=pixno])
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   key     Name of pixel-flag of the extension (STRING)
;
; OPTIONAL KEYWORDS:
;   PIXNO   FIS Pixel number (0-99 for SW, 0-74 for LW)
;               An integer or array expression is allowed.
;
; OUTPUTS:
;   valarray    Array of value (or pointer) in question
;
; DESCRIPTION:
;   Inquire flag for each pixel.  Flag keyword name are defined
;   separately. This function is practically equivalent with
;   Get_Value_TSD().
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Pix_Flag_TSD, extname, key, PIXNO=pixno
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Get_Pix_Flag(key, PIXNO=pixno)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Pix_Flag_TSD(extname, key, val [, PIXNO=pixno])
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2',
;                               'AOCU', 'GADS', 'PR', and 'SE')
;   key     Name of pixel-flag of the extension (STRING)
;   val     Value
;
; OPTIONAL KEYWORDS:
;   PIXNO   FIS Pixel number (0-99 for SW, 0-74 for LW)
;               An integer or array expression is allowed.
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Pix_Flag_TSD, extname, key, value, PIXNO=pixno
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Set_Pix_Flag(key, value, PIXNO=pixno)
END


;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   valarray = tsd->Get_Quality_TSD(extname, key [, PIXNO=pixno, BITS=bits])
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS' only)
;   key     Name of FIS Quality flag (STRING)
;
; OPTIONAL KEYWORDS:
;   PIXNO   FIS Pixel number (0-99 for SW, 0-74 for LW)
;               An integer or array expression is allowed.
;   BITS        Number of bits of quality is returned.
;
; OUTPUTS:
;   valarray    Array of value (or pointer) in question
;
; DESCRIPTION:
;   Inquire quality for each pixel.  Flag keyword name are defined
;   separately. This function is practically equivalent with
;   Get_Value_TSD().
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Get_Quality_TSD, extname, key, PIXNO=pixno, BITS=bits
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Get_Quality(key, PIXNO=pixno, BITS=bits)
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   status = tsd->Set_Quality_TSD(extname, key, val [, PIXNO=pixno, BITS=bits])
;
; INPUTS:
;   extname Name of FITS extension (STRING)
;               (now available: 'FIS_OBS' only)
;   key     Name of FIS Quality flag (STRING)
;   val     Value
;
; OPTIONAL KEYWORDS:
;   PIXNO   FIS Pixel number (0-99 for SW, 0-74 for LW)
;               An integer or array expression is allowed.
;   BITS        Number of bits of quality is returned.
;
; OUTPUTS:
;   status  Status of function (!SUCCEED or !FAIL)
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Set_Quality_TSD, extname, key, value, PIXNO=pixno, BITS=bits
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    p = self->getObjectPointer(extname)
    IF NOT PTR_VALID(p) THEN BEGIN
        MESSAGE, 'No such extname: ' + extname, /CONTINUE
        MESSAGE, 'Available extnames are: ' + $
                 STRJOIN(self->Get_Extnames(),','), /CONTINUE
        RETURN, !FAIL
    END
    RETURN, (*p)->Set_Quality(key, value, PIXNO=pixno, BITS=bits)
END


;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   subset = tsd->Subset_TSD(index)
;
; INPUTS:
;   index   Index array which specifies row numbers of original TSD
;
; OPTIONAL KEYWORDS:
;   None
;
; OUTPUTS:
;   subset  A TSD object instance which is newly created.  Specified
;               row index of original TSD (current instance) is copied.
;               Size of subset is equivalent to N_ELEMENTS(index).
;
; EXAMPLE:
;   You can get subset of current TSD between row# = 100-109 by entering:
;
;     IDL> offset = 100
;     IDL> index = INDGEN(10) + offset
;     IDL> subset = tsd->Subset_TSD(index)
;     IDL> dummy = subset->Write_TSD('output.fits')
;
;   You can check the subset TSD file (output.fits) as follows:
;
;     IDL> newtsd = OBJ_NEW('tsd')
;     IDL> dummy = newtsd->Read_TSD('output.fits')
;     IDL> aftime = newtsd->Get_Value_TSD('FIS_OBS', 'AFTIME')
;     IDL> help, aftime
;     AFTIME          INT       = Array[10]
;-
;----------------------------------------------------------------------------
FUNCTION tsd::Subset_TSD, index, VERSION=version
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    ; Check argument
    error = 0
    IF N_ELEMENTS(index) EQ 0 THEN error = 1
    tmp = WHERE(index LT 0, count)
    IF count NE 0 THEN error = 1
    sz = SIZE(index)
    IF (sz[sz[0]+1] NE 1) AND (sz[sz[0]+1] NE 2) AND (sz[sz[0]+1] NE 3) THEN error = 1
    IF error THEN BEGIN
        MESSAGE, 'Error: index seems to be something wrong.', /CONTINUE
        RETURN, !FAIL
    ENDIF

    ; Create a new instance to store subset of TSD
    subset = OBJ_NEW('tsd')
    d = subset->Create_TSD(self.datatype, N_ELEMENTS(index))
    ; Copy data to newly produced instance
    extnames = self->Get_Extnames(VERSION=version)
    FOR i=1,N_ELEMENTS(extnames) DO BEGIN
        newdata = (self.(i)->Get_Data())[index]
        dummy = subset.(i)->Set_Data(newdata)
        tmp_header = (self.(i)->Get_Header())
        dummy = subset.(i)->Set_Header(tmp_header)
    ENDFOR
    tmp_header = (self.(0)->Get_Header())
    dummy = subset.(0)->Set_Header(tmp_header)

    RETURN, subset
END


;----------------------------------------------------------------------------
; Not implemented yet...
;----------------------------------------------------------------------------

; Data search
FUNCTION tsd::Where_in_TSD, exp, count
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

;    cmd = 'idx = WHERE(' + exp + ', tsd)'
;    IF EXECUTE(cmd) THEN RETURN, TEMPORARY(idx) ELSE RETURN, ''
    RETURN, !FAIL
END

; Utilities
FUNCTION tsd::Concat_TSD, tsd1, tsd2
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    MESSAGE, 'tsd::Concat_TSD() is not implemented yet.'
    RETURN, !FAIL
;    RETURN, concat
END



;----------------------------------------------------------------------------
; Methods which are internal use only
;----------------------------------------------------------------------------

FUNCTION tsd::Get_ExtNames, VERSION=version
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    extnames = STRARR(1)
    ; Version of TSD structure definition.
    ; version 1: 2002/June -- 2003/Nov.
    ; version 2: 2003/Nov. -- 2004/Jun.
    ; version 3: 2004/Jun. -- 2006/Jan.
    ; version 4: 2006/Jan. -- current.
    tsd_version = 4
    IF KEYWORD_SET(version) THEN tsd_version = STRTRIM(STRING(version),2)
    CASE tsd_version OF
        '4': extnames = ['FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2', 'AOCU', 'GADS', 'PR', 'SE']
        '3': extnames = ['FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2', 'AOCU', 'GADS', 'PR', 'SE']
        '2': extnames = ['FIS_OBS', 'FIS_HK', 'IRC_HK', 'HK_2', 'AOCU', 'ADS', 'SE']
        '1': extnames = ['TSD_FIS', 'TSD_IRC', 'TSD_CRYO', 'TSD_AOCS', 'TSD_POINT', 'TSD_SE', 'TSD_HK']
        ELSE: MESSAGE, 'Unknown TSD version: "' + tsd_version + '"', /CONTINUE
    ENDCASE
    RETURN, extnames
END

FUNCTION tsd::createExtObj, VERSION=version
    COMPILE_OPT hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    CASE self.datatype OF
        'FIS_SW': fis_ndet = '100'
        'FIS_LW': fis_ndet = '75'
        'ISO': iso_ndet = '4'
        ELSE: MESSAGE, 'Unsupported datatype: "' + self.datatype + '"'
    ENDCASE

    ; Version of TSD structure definition.
    ; version 1: 2002/June -- 2003/Nov.
    ; version 2: 2003/Nov. -- 2004/Jun.
    ; version 3: 2004/Jun. -- 2006/Jan.
    ; version 4: 2006/Jan. -- current.
    tsd_version = 4
    IF KEYWORD_SET(version) THEN tsd_version = version
    CASE tsd_version OF
        4: tabledef_version4
        3: tabledef_version3
        2: tabledef_version2
        1: tabledef_version1
        ELSE: MESSAGE, 'Unknown TSD version: ' + tsd_version
    ENDCASE

    extnames = self->Get_Extnames(VERSION=tsd_version)
    self.fis_obs = OBJ_NEW('FIS_OBS', self.datatype, EXTVER=version)
    FOR i=2,N_ELEMENTS(extnames) DO BEGIN
        self.(i) = OBJ_NEW(extnames[i-1], EXTVER=version)
    ENDFOR

    RETURN, !TRUE
END

FUNCTION tsd::getObjectPointer, extname
    COMPILE_OPT hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    errmsg = ['Error (for TSD version 2):', $
              'The first argument of following methods should be an extension name.', $
              '  TSD::(Get|Set)_(Header|Header_Keyword|Value|Status|Flag|Pix_Flag)_TSD()', $
              '', $
              'Please rewrite your source code.']

    p = PTR_NEW()
    IF KEYWORD_SET(extname) THEN BEGIN
        i = 0
        IF NOT STRCMP(extname, 'PRIMARY', /FOLD_CASE) THEN BEGIN
            tsdver = FIX(self.primary->Get_Header_Keyword('FTYPEVER'))
            IF tsdver EQ 0 THEN tsdver = FIX(self.primary->Get_Header_Keyword('TSDVER'))
            IF tsdver EQ !FAIL THEN BEGIN
                STOP, 'ERROR: TSD object seems to be not initialized yet.'
            ENDIF
            extnames = self->Get_Extnames(VERSION=tsdver)
            FOR i=1,N_ELEMENTS(extnames) DO BEGIN
                IF STRCMP(extname, extnames[i-1], /FOLD_CASE) THEN BREAK
            ENDFOR
            IF i GT N_ELEMENTS(extnames) THEN BEGIN
;                MESSAGE, 'No such extname: ' + extname, /CONTINUE
;                MESSAGE, 'Available extnames are: ' + $
;                         STRJOIN(extnames,','), /CONTINUE
                IF tsdver LT 2 THEN MESSAGE, errmsg, /CONTINUE
;                HELP, /TRACEBACK
                RETURN, PTR_NEW()
            ENDIF
        ENDIF
        p = PTR_NEW(self.(i))
    ENDIF ELSE BEGIN
        p = PTR_NEW(self.primary)
    ENDELSE
    RETURN, p
END

FUNCTION tsd::URLencode, str
    COMPILE_OPT hidden
    ; encode except  A-Z,a-z,0-9,'.-*)(_
    encode_char = [' ', $
                   '!',  '"',  '#',  '$',  '%',  '&',  ',',  ':', $
                   ';',  '<',  '=',  '>',  '?',  '@',  '[',  '\', $
                   ']',  '^',  '_',  '`',  '{',  '|',  '}',  '~'  ]
    encode_str  = ['+', $
                   '%21','%22','%23','%24','%25','%26','%2c','%3a', $
                   '%3b','%3c','%3d','%3e','%3f','%40','%5b','%5c', $
                   '%5d','%5e','%5f','%60','%7b','%7c','%7d','%7e'  ]

    newstr = str
    FOR i=0,N_ELEMENTS(encode_char)-1 DO BEGIN
        IF STRPOS(str, encode_char[i]) NE -1 THEN BEGIN
            tmparr = STRSPLIT(newstr, encode_char[i], /EXTRACT)
            newstr = STRJOIN(tmparr, encode_str[i])
        ENDIF
    ENDFOR
    RETURN, newstr
END

; psuedo_tmpnam - create a name for a temporary file
FUNCTION tsd::psuedo_tmpnam
    COMPILE_OPT hidden
    chars = ['a','b','c','d','e','f','g','h','i','j','k','l','m',$
             'n','o','p','q','r','s','t','u','v','w','x','y','z',$
             'A','B','C','D','E','F','G','H','I','J','K','L','M',$
             'N','O','P','Q','R','S','T','U','V','W','X','Y','Z']
    nStr = 10
    seed = SYSTIME(/SECONDS)
    index = FIX(RANDOMU(seed, nStr) * N_ELEMENTS(chars))
    RETURN, STRJOIN(chars[index], '')
END



;----------------------------------------------------------------------------
; Methods which are obsoleted
;----------------------------------------------------------------------------

FUNCTION tsd::getProperty, key, EXTNAME=extname
    COMPILE_OPT hidden
    MESSAGE, "Function tsd->getProperty(key) is obsoleted.", /CONTINUE
    MESSAGE, "Use tsd->Get_Header_Keyword_TSD(key) instead.", /CONTINUE
    RETURN, self->Get_Header_Keyword_TSD(extname, key)
END

FUNCTION tsd::setProperty, key, val, EXTNAME=extname
    COMPILE_OPT hidden
    MESSAGE, "Function tsd->setProperty(key,val) is obsoleted.", /CONTINUE
    MESSAGE, "Use tsd->Get_Header_Keyword_TSD(key,val) instead.", /CONTINUE
    RETURN, self->Set_Header_Keyword_TSD(extname, key, val)
END


;----------------------------------------------------------------------------
; Methods to define the class structure
;----------------------------------------------------------------------------

; INIT should return a scalar TRUE value if the initialization
; is successful, and FALSE if the initialization fails.
FUNCTION tsd::INIT, DATATYPE=datatype, INFILE=infile
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    IF !VERSION.RELEASE LT '5.6' THEN STOP, 'Should use after IDL ver.5.6'
    IF LMGR(/DEMO) THEN MESSAGE, 'IDL works on DEMO mode.  Some features of TSD interfaces (such as Write_TSD()) are restricted.', /INFORMATIONAL
    IF LMGR(/VM) THEN MESSAGE, 'IDL works on VM mode.  Some features of TSD interfaces (such as Read_TSD_From_LDS()) are restricted.', /INFORMATIONAL

    self.primary   = OBJ_NEW('primary')
    IF KEYWORD_SET(datatype) THEN BEGIN
        self.datatype = STRUPCASE(STRTRIM(datatype,2))
        dummy = self->createExtObj()
    ENDIF
    IF KEYWORD_SET(infile) THEN BEGIN
        dummy = self->Read_TSD(infile)
    ENDIF

    RETURN, !TRUE
END

; Free all heap variables recursively.
; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
PRO tsd::CLEANUP
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    extnames = self->Get_ExtNames()
    FOR i=0,N_ELEMENTS(extnames) DO BEGIN
        OBJ_DESTROY, self.(i)
    ENDFOR

    HEAP_GC ;, /VERBOSE
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   tsdobj = OBJ_NEW('tsd' [, DATATYPE=datatype] [, INFILE='infile.fits'])
;
; INPUTS:
;   'tsd'   Name of object class
;
; OPTIONAL KEYWORD:
;   DATATYPE    Datatype (STRING)
;               (now available: 'FIS_LW' and 'FIS_SW')
;   INFILE  Input filename of TSD (which is a FITS format) (STRING)
;
; OUTPUTS:
;   tsdobj  Name of object instance
;
; EXAMPLE:
;   IDL> tsdobj=OBJ_NEW('tsd')
;   IDL> help, tsdobj
;   TSDOBJ          OBJREF    = <ObjHeapVar1(TSD)>
;-
;----------------------------------------------------------------------------
PRO tsd__define
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    ; Compile utility functions at first.
    RESOLVE_ROUTINE, 'util'
    RESOLVE_ROUTINE, 'tabledef'
    util
    setDebug
;    beVerbose

    STRUCT_HIDE, { tsd, $ ; structure name
          primary:  OBJ_NEW(), $
          fis_obs:  OBJ_NEW(), $
          fis_hk:   OBJ_NEW(), $
          irc_hk:   OBJ_NEW(), $
          hk_2:     OBJ_NEW(), $
          aocu:     OBJ_NEW(), $
          gads:     OBJ_NEW(), $
          pr:       OBJ_NEW(), $
          se:       OBJ_NEW(), $
          filename: '', $
          instrume: '', $
          datatype: ''  }
END

;----------------------------------------------------------------------------
; End of file
;----------------------------------------------------------------------------
